@@ -7,10 +7,11 @@
 #include "allegro5/allegro.h"
 #include "allegro5/allegro_acodec.h"
 #include "allegro5/allegro_audio.h"
-#include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_audio.h"
+#include "allegro5/internal/aintern_exitfunc.h"
 #include "allegro5/internal/aintern_system.h"
 #include "acodec.h"
+#include "helper.h"
 
 #ifndef ALLEGRO_CFG_ACODEC_MODAUDIO
    #error configuration problem, ALLEGRO_CFG_ACODEC_MODAUDIO not set
@@ -131,7 +132,8 @@ static size_t modaudio_stream_update(ALLEGRO_AUDIO_STREAM *stream, void *data,
 static void modaudio_stream_close(ALLEGRO_AUDIO_STREAM *stream)
 {
    MOD_FILE *const df = stream->extra;
-   
+   _al_acodec_stop_feed_thread(stream);
+      
    lib.duh_end_sigrenderer(df->sig);
    lib.unload_duh(df->duh);
    if (df->fh)
