@@ -22,32 +22,62 @@
 #include <pulse/error.h>
 #include <pulse/introspect.h>
 #include <pulse/mainloop.h>
-#include <malloc.h>
+#include <stdlib.h>
 
 ALLEGRO_DEBUG_CHANNEL("PulseAudio")
 
+enum PULSEAUDIO_VOICE_STATUS {
+   PV_IDLE,
+   PV_PLAYING,
+   PV_STOPPING,
+   PV_JOIN
+};
+
 typedef struct PULSEAUDIO_VOICE
 {
-  pa_simple *s;
-  ALLEGRO_THREAD *poll_thread;
+   pa_simple *s;
+   unsigned int buffer_size_in_frames;
+   unsigned int frame_size_in_bytes;
 
-  volatile enum { PV_PLAYING, PV_STOPPING, PV_STOPPED } status;
+   ALLEGRO_THREAD *poll_thread;
+   ALLEGRO_MUTEX *status_mutex;
+   ALLEGRO_COND *status_cond;
+   enum PULSEAUDIO_VOICE_STATUS status;
 
-  int frame_size; // number of bytes per frame 
-
-  // direct buffer (non-streaming):
-  ALLEGRO_MUTEX *buffer_mutex;  
-  char *buffer, *buffer_end;
-   
+   // direct buffer (non-streaming):
+   ALLEGRO_MUTEX *buffer_mutex;  
+   char *buffer;
+   char *buffer_end;
 } PULSEAUDIO_VOICE;
 
+#define DEFAULT_BUFFER_SIZE   1024
+#define MIN_BUFFER_SIZE       128
+
+static unsigned int get_buffer_size(const ALLEGRO_CONFIG *config)
+{
+   if (config) {
+      const char *val = al_get_config_value(config,
+         "pulseaudio", "buffer_size");
+      if (val && val[0] != '\0') {
+         int n = atoi(val);
+         if (n < MIN_BUFFER_SIZE)
+            n = MIN_BUFFER_SIZE;
+         return n;
+      }
+   }
+
+   return DEFAULT_BUFFER_SIZE;
+}
+
 static void sink_info_cb(pa_context *c, const pa_sink_info *i, int eol,
    void *userdata)
 {
    (void)c;
    (void)eol;
+
    pa_sink_state_t *ret = userdata;
-   if (!i) return;
+   if (!i)
+      return;
    *ret = i->state;
 }
 
@@ -75,24 +105,30 @@ static int pulseaudio_open(void)
 
    pa_context_connect(c, NULL, PA_CONTEXT_NOAUTOSPAWN, NULL);
 
-   // TODO: We could set a timeout here if PA decides to try connecting
-   // forever.
    while (1) {
-      pa_mainloop_iterate(mainloop, 1, NULL);
+      /* Don't block or it will hang if there is no server to connect to. */
+      const int blocking = 0;
+      if (pa_mainloop_iterate(mainloop, blocking, NULL) < 0) {
+         ALLEGRO_ERROR("pa_mainloop_iterate failed\n");
+         pa_context_disconnect(c);
+         pa_mainloop_free(mainloop);
+         break;
+      }
       pa_context_state_t s = pa_context_get_state(c);
+      if (s == PA_CONTEXT_READY) {
+         ALLEGRO_DEBUG("PA_CONTEXT_READY\n");
+         break;
+      }
       if (s == PA_CONTEXT_FAILED) {
+         ALLEGRO_ERROR("PA_CONTEXT_FAILED\n");
          pa_context_disconnect(c);
          pa_mainloop_free(mainloop);
          return 1;
       }
-      if (s == PA_CONTEXT_READY ) {
-         break;
-      }
    }
-   
+
    pa_sink_state_t state = 0;
-   pa_operation *op = pa_context_get_sink_info_list(c, sink_info_cb,
-      &state);
+   pa_operation *op = pa_context_get_sink_info_list(c, sink_info_cb, &state);
    while (pa_operation_get_state(op) == PA_OPERATION_RUNNING) {
       pa_mainloop_iterate(mainloop, 1, NULL);
    }
@@ -116,29 +152,46 @@ static void *pulseaudio_update(ALLEGRO_THREAD *self, void *data)
 {
    ALLEGRO_VOICE *voice = data;
    PULSEAUDIO_VOICE *pv = voice->extra;
+   (void)self;
 
-   while (!al_get_thread_should_stop(self)) {
-      if (pv->status == PV_PLAYING) {
-         unsigned int frames = 4096;
+   for (;;) {
+      enum PULSEAUDIO_VOICE_STATUS status;
+
+      al_lock_mutex(pv->status_mutex);
+      while ((status = pv->status) == PV_IDLE) {
+         al_wait_cond(pv->status_cond, pv->status_mutex);
+      }
+      al_unlock_mutex(pv->status_mutex);
+
+      if (status == PV_JOIN) {
+         break;
+      }
+
+      if (status == PV_PLAYING) {
+         unsigned int frames = pv->buffer_size_in_frames;
          if (voice->is_streaming) { 
             // streaming audio           
             const void *data = _al_voice_update(voice, &frames);
             if (data) {
-               pa_simple_write(pv->s, data, frames * pv->frame_size, NULL);
+               pa_simple_write(pv->s, data,
+                  frames * pv->frame_size_in_bytes, NULL);
             }
          }
          else {
             // direct buffer audio
             al_lock_mutex(pv->buffer_mutex);
             const char *data = pv->buffer;
-            unsigned int len = frames * pv->frame_size;
-            pv->buffer += frames * pv->frame_size;
+            unsigned int len = frames * pv->frame_size_in_bytes;
+            pv->buffer += frames * pv->frame_size_in_bytes;
             if (pv->buffer > pv->buffer_end) {
                len = pv->buffer_end - data;
                pv->buffer = voice->attached_stream->spl_data.buffer.ptr;
                voice->attached_stream->pos = 0;
                if (voice->attached_stream->loop == ALLEGRO_PLAYMODE_ONCE) {
+                  al_lock_mutex(pv->status_mutex);
                   pv->status = PV_STOPPING;
+                  al_broadcast_cond(pv->status_cond);
+                  al_unlock_mutex(pv->status_mutex);
                }
             }
             else {
@@ -149,12 +202,12 @@ static void *pulseaudio_update(ALLEGRO_THREAD *self, void *data)
             pa_simple_write(pv->s, data, len, NULL);
          }
       }
-      else if (pv->status == PV_STOPPING) {
+      else if (status == PV_STOPPING) {
          pa_simple_flush(pv->s, NULL);
-         pv->status = PV_STOPPED;
-      }
-      else if (pv->status == PV_STOPPED) {
-         al_rest(0.001);
+         al_lock_mutex(pv->status_mutex);
+         pv->status = PV_IDLE;
+         al_broadcast_cond(pv->status_cond);
+         al_unlock_mutex(pv->status_mutex);
       }
    }
 
@@ -192,15 +245,16 @@ static int pulseaudio_allocate_voice(ALLEGRO_VOICE *voice)
    ba.minreq    = 0;       // minimum size of request 
    ba.fragsize  = -1;      // fragment size (recording)
 
-   pv->s = pa_simple_new(NULL,         // Use the default server.
-                   al_get_app_name(),     
-                   PA_STREAM_PLAYBACK,
-                   NULL,               // Use the default device.
-                   "Allegro Voice",    
-                   &ss,                
-                   NULL,               // Use default channel map
-                   &ba,                
-                   NULL                // Ignore error code.
+   pv->s = pa_simple_new(
+      NULL,                // Use the default server.
+      al_get_app_name(),     
+      PA_STREAM_PLAYBACK,
+      NULL,                // Use the default device.
+      "Allegro Voice",    
+      &ss,                
+      NULL,                // Use default channel map
+      &ba,                
+      NULL                 // Ignore error code.
    );
 
    if (!pv->s) {
@@ -210,8 +264,12 @@ static int pulseaudio_allocate_voice(ALLEGRO_VOICE *voice)
 
    voice->extra = pv;
 
-   pv->frame_size = ss.channels * al_get_audio_depth_size(voice->depth);
-   pv->status = PV_STOPPED;
+   pv->buffer_size_in_frames = get_buffer_size(al_get_system_config());
+   pv->frame_size_in_bytes = ss.channels * al_get_audio_depth_size(voice->depth);
+
+   pv->status = PV_IDLE;
+   pv->status_mutex = al_create_mutex();
+   pv->status_cond = al_create_cond();
    pv->buffer_mutex = al_create_mutex();
 
    pv->poll_thread = al_create_thread(pulseaudio_update, (void*)voice);
@@ -224,13 +282,19 @@ static void pulseaudio_deallocate_voice(ALLEGRO_VOICE *voice)
 {
    PULSEAUDIO_VOICE *pv = voice->extra;
 
+   al_lock_mutex(pv->status_mutex);
+   pv->status = PV_JOIN;
+   al_broadcast_cond(pv->status_cond);
+   al_unlock_mutex(pv->status_mutex);
+
    /* We do NOT hold the voice mutex here, so this does NOT result in a
     * deadlock when the thread calls _al_voice_update.
     */
-   al_set_thread_should_stop(pv->poll_thread);
    al_join_thread(pv->poll_thread, NULL);
    al_destroy_thread(pv->poll_thread);
 
+   al_destroy_mutex(pv->status_mutex);
+   al_destroy_cond(pv->status_cond);
    al_destroy_mutex(pv->buffer_mutex);
 
    pa_simple_free(pv->s);
@@ -240,6 +304,7 @@ static void pulseaudio_deallocate_voice(ALLEGRO_VOICE *voice)
 static int pulseaudio_load_voice(ALLEGRO_VOICE *voice, const void *data)
 {
    PULSEAUDIO_VOICE *pv = voice->extra;
+   (void)data;
 
    if (voice->attached_stream->loop == ALLEGRO_PLAYMODE_BIDIR) {
       ALLEGRO_INFO("Backwards playing not supported by the driver.\n");
@@ -249,10 +314,10 @@ static int pulseaudio_load_voice(ALLEGRO_VOICE *voice, const void *data)
    voice->attached_stream->pos = 0;
 
    pv->buffer = voice->attached_stream->spl_data.buffer.ptr;
-   pv->buffer_end = pv->buffer + (voice->attached_stream->spl_data.len) * pv->frame_size;
+   pv->buffer_end = pv->buffer +
+      (voice->attached_stream->spl_data.len) * pv->frame_size_in_bytes;
 
    return 0;
-   (void)data;
 }
 
 static void pulseaudio_unload_voice(ALLEGRO_VOICE *voice)
@@ -263,27 +328,52 @@ static void pulseaudio_unload_voice(ALLEGRO_VOICE *voice)
 static int pulseaudio_start_voice(ALLEGRO_VOICE *voice)
 {
    PULSEAUDIO_VOICE *pv = voice->extra;   
-   pv->status = PV_PLAYING;
-   
-   return 0;
+   int ret;
+
+   al_lock_mutex(pv->status_mutex);
+   if (pv->status == PV_IDLE) {
+      pv->status = PV_PLAYING;
+      al_broadcast_cond(pv->status_cond);
+      ret = 0;
+   }
+   else {
+      ret = 1;
+   }
+   al_unlock_mutex(pv->status_mutex);
+
+   return ret;
 }
 
 static int pulseaudio_stop_voice(ALLEGRO_VOICE *voice)
 {
    PULSEAUDIO_VOICE *pv = voice->extra;
-   pv->status = PV_STOPPING;
 
-   while (pv->status == PV_STOPPING) {
-      al_rest(0.001);
+   al_lock_mutex(pv->status_mutex);
+
+   if (pv->status == PV_PLAYING) {
+      pv->status = PV_STOPPING;
+      al_broadcast_cond(pv->status_cond);
    }
-   
+
+   while (pv->status != PV_IDLE) {
+      al_wait_cond(pv->status_cond, pv->status_mutex);
+   }
+
+   al_unlock_mutex(pv->status_mutex);
+
    return 0;
 }
 
 static bool pulseaudio_voice_is_playing(const ALLEGRO_VOICE *voice)
 {
    PULSEAUDIO_VOICE *pv = voice->extra;
-   return pv->status == PV_PLAYING;
+   enum PULSEAUDIO_VOICE_STATUS status;
+
+   al_lock_mutex(pv->status_mutex);
+   status = pv->status;
+   al_unlock_mutex(pv->status_mutex);
+
+   return (status == PV_PLAYING);
 }
 
 static unsigned int pulseaudio_get_voice_position(const ALLEGRO_VOICE *voice)
@@ -296,10 +386,11 @@ static int pulseaudio_set_voice_position(ALLEGRO_VOICE *voice, unsigned int pos)
    PULSEAUDIO_VOICE *pv = voice->extra;
 
    pa_simple_drain(pv->s, NULL);
-   
+
    al_lock_mutex(pv->buffer_mutex);
    voice->attached_stream->pos = pos;
-   pv->buffer = (char*)voice->attached_stream->spl_data.buffer.ptr + pos * pv->frame_size;
+   pv->buffer = (char *)voice->attached_stream->spl_data.buffer.ptr +
+      pos * pv->frame_size_in_bytes;
    al_unlock_mutex(pv->buffer_mutex);
 
    return 0;
@@ -326,3 +417,5 @@ ALLEGRO_AUDIO_DRIVER _al_kcm_pulseaudio_driver =
    pulseaudio_get_voice_position,
    pulseaudio_set_voice_position
 };
+
+/* vim: set sts=3 sw=3 et: */
