@@ -1,9 +1,39 @@
 #include "allegro5/allegro.h"
 #include "allegro5/allegro_native_dialog.h"
-#include "allegro5/internal/aintern_native_dialog.h"
 #include "allegro5/internal/aintern_dtor.h"
+#include "allegro5/internal/aintern_exitfunc.h"
+#include "allegro5/internal/aintern_native_dialog.h"
 #include "allegro5/internal/aintern_system.h"
 
+ALLEGRO_DEBUG_CHANNEL("native_dialog")
+
+static bool inited_addon = false;
+
+/* Function: al_init_native_dialog_addon
+ */
+bool al_init_native_dialog_addon(void)
+{
+   if (!inited_addon) {
+      if (!_al_init_native_dialog_addon()) {
+         ALLEGRO_ERROR("_al_init_native_dialog_addon failed.\n");
+         return false;
+      }
+      inited_addon = true;
+      _al_add_exit_func(al_shutdown_native_dialog_addon,
+         "al_shutdown_native_dialog_addon");
+   }
+   return true;
+}
+
+/* Function: al_shutdown_native_dialog_addon
+ */
+void al_shutdown_native_dialog_addon(void)
+{
+   if (inited_addon) {
+      _al_shutdown_native_dialog_addon();
+      inited_addon = false;
+   }
+}
 
 /* Function: al_create_native_file_dialog
  */
