@@ -170,6 +170,9 @@ int main(void)
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
    }
+   if (!al_init_native_dialog_addon()) {
+      abort_example("Could not init native dialog addon.\n");
+   }
 
    textlog = al_open_native_text_log("Log", 0);
    message("Starting up log window.\n");
@@ -191,7 +194,6 @@ int main(void)
    if (!display) {
       message("failure.\n");
       abort_example("Error creating display\n");
-      return 1;
    }
    message("success.\n");
 
@@ -200,7 +202,6 @@ int main(void)
    if (!font) {
       message("failure.\n");
       abort_example("Error loading data/fixed_font.tga\n");
-      return 1;
    }
    message("success.\n");
 
