@@ -75,8 +75,7 @@ Widget::Widget():
    x1(0),
    y1(0),
    x2(0),
-   y2(0),
-   disabled(false)
+   y2(0)
 {
 }
 
@@ -116,10 +115,6 @@ Dialog::Dialog(const Theme & theme, ALLEGRO_DISPLAY *display,
    al_register_event_source(this->event_queue, al_get_keyboard_event_source());
    al_register_event_source(this->event_queue, al_get_mouse_event_source());
    al_register_event_source(this->event_queue, al_get_display_event_source(display));
-   if (al_is_touch_input_installed()) {
-      al_register_event_source(this->event_queue,
-         al_get_touch_input_mouse_emulation_event_source());
-   }
 }
 
 Dialog::~Dialog()
@@ -246,16 +241,16 @@ void Dialog::on_mouse_axes(const ALLEGRO_MOUSE_EVENT & event)
       return;
    }
 
-   this->check_mouse_over(mx, my);
-}
-
-void Dialog::check_mouse_over(int mx, int my)
-{
    if (this->mouse_over_widget && this->mouse_over_widget->contains(mx, my)) {
       /* no change */
       return;
    }
 
+   this->check_mouse_over(mx, my);
+}
+
+void Dialog::check_mouse_over(int mx, int my)
+{
    for (std::list<Widget*>::iterator it = this->all_widgets.begin();
          it != this->all_widgets.end();
          ++it)
@@ -277,11 +272,6 @@ void Dialog::on_mouse_button_down(const ALLEGRO_MOUSE_EVENT & event)
 {
    if (event.button != 1)
       return;
-
-   /* With touch input we may not receive mouse axes event before the touch
-    * so we must check which widget the touch is over.
-    */
-   this->check_mouse_over(event.x, event.y);
    if (!this->mouse_over_widget)
       return;
 
@@ -381,19 +371,14 @@ void Label::draw()
 {
    const Theme & theme = this->dialog->get_theme();
    SaveState state;
-   ALLEGRO_COLOR fg = theme.fg;
-
-   if (is_disabled()) {
-      fg = al_map_rgb(64, 64, 64);
-   }
 
    al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
    if (centred) {
-      al_draw_text(theme.font, fg, (this->x1 + this->x2 + 1)/2,
+      al_draw_text(theme.font, theme.fg, (this->x1 + this->x2 + 1)/2,
          this->y1, ALLEGRO_ALIGN_CENTRE, this->text.c_str());
    }
    else {
-      al_draw_text(theme.font, fg, this->x1, this->y1, 0, this->text.c_str());
+      al_draw_text(theme.font, theme.fg, this->x1, this->y1, 0, this->text.c_str());
    }
 }
 
@@ -419,10 +404,6 @@ void Button::on_mouse_button_down(int mx, int my)
 {
    (void)mx;
    (void)my;
-
-   if (is_disabled())
-      return;
-
    this->pushed = true;
    dialog->request_draw();
 }
@@ -431,10 +412,6 @@ void Button::on_mouse_button_up(int mx, int my)
 {
    (void)mx;
    (void)my;
-
-   if (is_disabled())
-      return;
-
    this->pushed = false;
    dialog->request_draw();
 }
@@ -445,7 +422,6 @@ void Button::draw()
    ALLEGRO_COLOR fg;
    ALLEGRO_COLOR bg;
    SaveState state;
-   double y;
 
    if (this->pushed) {
       fg = theme.bg;
@@ -456,23 +432,13 @@ void Button::draw()
       bg = theme.bg;
    }
 
-   if (is_disabled()) {
-      bg = al_map_rgb(64, 64, 64);
-   }
-
    al_draw_filled_rectangle(this->x1, this->y1,
       this->x2, this->y2, bg);
    al_draw_rectangle(this->x1 + 0.5, this->y1 + 0.5,
       this->x2 - 0.5, this->y2 - 0.5, fg, 0);
    al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
-
-   /* Center the text vertically in the button, taking the font size
-    * into consideration.
-    */
-   y = (this->y1 + this->y2 - al_get_font_line_height(theme.font) - 1) / 2;
-
    al_draw_text(theme.font, fg, (this->x1 + this->x2 + 1)/2,
-      y, ALLEGRO_ALIGN_CENTRE, this->text.c_str());
+      this->y1, ALLEGRO_ALIGN_CENTRE, this->text.c_str());
 }
 
 bool Button::get_pushed()
@@ -491,10 +457,6 @@ void ToggleButton::on_mouse_button_down(int mx, int my)
 {
    (void)mx;
    (void)my;
-
-   if (is_disabled())
-      return;
-
    set_pushed(!this->pushed);
 }
 
@@ -502,9 +464,6 @@ void ToggleButton::on_mouse_button_up(int mx, int my)
 {
    (void)mx;
    (void)my;
-
-   if (is_disabled())
-      return;
 }
 
 void ToggleButton::set_pushed(bool pushed)
@@ -527,14 +486,11 @@ List::List(int initial_selection) :
 
 bool List::want_key_focus()
 {
-   return !is_disabled();
+   return true;
 }
 
 void List::on_key_down(const ALLEGRO_KEYBOARD_EVENT & event)
 {
-   if (is_disabled())
-      return;
-
    switch (event.keycode) {
       case ALLEGRO_KEY_DOWN:
          if (selected_item < items.size() - 1) {
@@ -554,9 +510,6 @@ void List::on_key_down(const ALLEGRO_KEYBOARD_EVENT & event)
 
 void List::on_click(int mx, int my)
 {
-   if (is_disabled())
-      return;
-
    const Theme & theme = dialog->get_theme();
    unsigned int i = (my - this->y1) / al_get_font_line_height(theme.font);
    if (i < this->items.size()) {
@@ -572,13 +525,8 @@ void List::draw()
 {
    const Theme & theme = dialog->get_theme();
    SaveState state;
-   ALLEGRO_COLOR bg = theme.bg;
 
-   if (is_disabled()) {
-      bg = al_map_rgb(64, 64, 64);
-   }
-
-   al_draw_filled_rectangle(x1 + 1, y1 + 1, x2 - 1, y2 - 1, bg);
+   al_draw_filled_rectangle(x1 + 1, y1 + 1, x2 - 1, y2 - 1, theme.bg);
 
    al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
    const int font_height = al_get_font_line_height(theme.font);
@@ -628,17 +576,11 @@ VSlider::VSlider(int cur_value, int max_value) :
 
 void VSlider::on_mouse_button_down(int mx, int my)
 {
-   if (is_disabled())
-      return;
-
    this->on_mouse_button_hold(mx, my);
 }
 
 void VSlider::on_mouse_button_hold(int mx, int my)
 {
-   if (is_disabled())
-      return;
-
    double r = (double) (this->y2 - 1 - my) / (this->height() - 2);
    r = CLAMP(0.0, r, 1.0);
    cur_value = (int) (r * max_value);
@@ -650,16 +592,11 @@ void VSlider::on_mouse_button_hold(int mx, int my)
 void VSlider::draw()
 {
    const Theme & theme = dialog->get_theme();
-   ALLEGRO_COLOR bg = theme.fg;
    float left = x1 + 0.5, top = y1 + 0.5;
    float right = x2 + 0.5, bottom = y2 + 0.5;
    SaveState state;
 
-   if (is_disabled()) {
-      bg = al_map_rgb(64, 64, 64);
-   }
-
-   al_draw_rectangle(left, top, right, bottom, bg, 1);
+   al_draw_rectangle(left, top, right, bottom, theme.fg, 1);
 
    double ratio = (double) this->cur_value / (double) this->max_value;
    int ypos = (int) (bottom - 0.5 - (int) (ratio * (height() - 7)));
@@ -671,11 +608,6 @@ int VSlider::get_cur_value() const
    return this->cur_value;
 }
 
-int VSlider::get_max_value() const
-{
-   return this->max_value;
-}
-
 void VSlider::set_cur_value(int v)
 {
    this->cur_value = v;
@@ -691,17 +623,11 @@ HSlider::HSlider(int cur_value, int max_value) :
 
 void HSlider::on_mouse_button_down(int mx, int my)
 {
-   if (is_disabled())
-      return;
-
    this->on_mouse_button_hold(mx, my);
 }
 
 void HSlider::on_mouse_button_hold(int mx, int my)
 {
-   if (is_disabled())
-      return;
-
    double r = (double) (mx - 1 - this->x1) / (this->width() - 2);
    r = CLAMP(0.0, r, 1.0);
    cur_value = (int) (r * max_value);
@@ -715,13 +641,8 @@ void HSlider::draw()
    const Theme & theme = dialog->get_theme();
    const int cy = (y1 + y2) / 2;
    SaveState state;
-   ALLEGRO_COLOR bg = theme.bg;
 
-   if (is_disabled()) {
-      bg = al_map_rgb(64, 64, 64);
-   }
-
-   al_draw_filled_rectangle(x1, y1, x2, y2, bg);
+   al_draw_filled_rectangle(x1, y1, x2, y2, theme.bg);
    al_draw_line(x1, cy, x2, cy, theme.fg, 0);
 
    double ratio = (double) this->cur_value / (double) this->max_value;
@@ -734,11 +655,6 @@ int HSlider::get_cur_value() const
    return this->cur_value;
 }
 
-int HSlider::get_max_value() const
-{
-   return this->max_value;
-}
-
 void HSlider::set_cur_value(int v)
 {
    this->cur_value = v;
@@ -761,7 +677,7 @@ TextEntry::~TextEntry()
 
 bool TextEntry::want_key_focus()
 {
-   return !is_disabled();
+   return true;
 }
 
 void TextEntry::got_key_focus()
@@ -778,9 +694,6 @@ void TextEntry::lost_key_focus()
 
 void TextEntry::on_key_down(const ALLEGRO_KEYBOARD_EVENT & event)
 {
-   if (is_disabled())
-      return;
-
    switch (event.keycode) {
       case ALLEGRO_KEY_LEFT:
          al_ustr_prev(text, &cursor_pos);
@@ -845,13 +758,8 @@ void TextEntry::draw()
 {
    const Theme & theme = dialog->get_theme();
    SaveState state;
-   ALLEGRO_COLOR bg = theme.bg;
 
-   if (is_disabled()) {
-      bg = al_map_rgb(64, 64, 64);
-   }
-
-   al_draw_filled_rectangle(x1, y1, x2, y2, bg);
+   al_draw_filled_rectangle(x1, y1, x2, y2, theme.bg);
 
    al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
 
