@@ -55,7 +55,7 @@
 
 #define ALLEGRO_VERSION          5
 #define ALLEGRO_SUB_VERSION      0
-#define ALLEGRO_WIP_VERSION      8
+#define ALLEGRO_WIP_VERSION      10
 
 /* Not sure we need it, but since ALLEGRO_VERSION_STR contains it:
  * 0 = SVN
@@ -67,9 +67,9 @@
  */
 #define ALLEGRO_RELEASE_NUMBER   1
 
-#define ALLEGRO_VERSION_STR      "5.0.8"
-#define ALLEGRO_DATE_STR         "2012"
-#define ALLEGRO_DATE             20121118    /* yyyymmdd */
+#define ALLEGRO_VERSION_STR      "5.0.10"
+#define ALLEGRO_DATE_STR         "2013"
+#define ALLEGRO_DATE             20130616    /* yyyymmdd */
 #define ALLEGRO_VERSION_INT \
     ((ALLEGRO_VERSION << 24) | (ALLEGRO_SUB_VERSION << 16) | \
     (ALLEGRO_WIP_VERSION << 8) | ALLEGRO_RELEASE_NUMBER)
@@ -87,13 +87,6 @@ AL_FUNC(int, al_run_main, (int argc, char **argv, int (*)(int, char **)));
 
 #define AL_ID(a,b,c,d)     (((a)<<24) | ((b)<<16) | ((c)<<8) | (d))
 
-typedef struct _AL_DRIVER_INFO      /* info about a hardware driver */
-{
-   int id;                          /* integer ID */
-   void *driver;                    /* the driver structure */
-   int autodetect;                  /* set to allow autodetection */
-} _AL_DRIVER_INFO;
-
        
 
 #ifdef __cplusplus
