@@ -24,7 +24,8 @@
 
 #include "allegro5/allegro.h"
 #include "allegro5/internal/aintern.h"
-#include ALLEGRO_INTERNAL_HEADER
+#include "allegro5/internal/aintern_bitmap.h"
+#include "allegro5/internal/aintern_display.h"
 #include "allegro5/internal/aintern_system.h"
 #include "allegro5/internal/aintern_display.h"
 #include "allegro5/internal/aintern_bitmap.h"
@@ -44,10 +45,14 @@ ALLEGRO_DISPLAY *al_create_display(int w, int h)
 
    system = al_get_system_driver();
    driver = system->vt->get_display_driver();
+   if (!driver) {
+      ALLEGRO_ERROR("Failed to create display (no display driver)\n");
+      return NULL;
+   }
+
    display = driver->create_display(w, h);
-   
    if (!display) {
-      ALLEGRO_DEBUG("Failed to create display (NULL)\n");
+      ALLEGRO_ERROR("Failed to create display (NULL)\n");
       return NULL;
    }
 
@@ -193,48 +198,6 @@ bool al_resize_display(ALLEGRO_DISPLAY *display, int width, int height)
 
 
 
-/* Function: al_clear_to_color
- */
-void al_clear_to_color(ALLEGRO_COLOR color)
-{
-   ALLEGRO_BITMAP *target = al_get_target_bitmap();
-   ASSERT(target);
-
-   if (target->flags & ALLEGRO_MEMORY_BITMAP) {
-      _al_clear_memory(&color);
-   }
-   else {
-      ALLEGRO_DISPLAY *display = target->display;
-      ASSERT(display);
-      ASSERT(display->vt);
-      display->vt->clear(display, &color);
-   }
-}
-
-
-
-/* Function: al_draw_pixel
- */
-void al_draw_pixel(float x, float y, ALLEGRO_COLOR color)
-{
-   ALLEGRO_BITMAP *target = al_get_target_bitmap();
-
-   ASSERT(target);
-
-   if (target->flags & ALLEGRO_MEMORY_BITMAP) {
-      _al_draw_pixel_memory(target, x, y, &color);
-   }
-   else {
-      ALLEGRO_DISPLAY *display = target->display;
-      ASSERT(display);
-      ASSERT(display->vt);
-      display->vt->draw_pixel(display, x, y, &color);
-   }
-}
-
-
-
-
 /* Function: al_is_compatible_bitmap
  */
 bool al_is_compatible_bitmap(ALLEGRO_BITMAP *bitmap)
@@ -305,26 +268,6 @@ int al_get_display_flags(ALLEGRO_DISPLAY *display)
 
 
 
-/* Function: al_get_num_display_modes
- */
-int al_get_num_display_modes(void)
-{
-   ALLEGRO_SYSTEM *system = al_get_system_driver();
-   return system->vt->get_num_display_modes();
-}
-
-
-
-/* Function: al_get_display_mode
- */
-ALLEGRO_DISPLAY_MODE *al_get_display_mode(int index, ALLEGRO_DISPLAY_MODE *mode)
-{
-   ALLEGRO_SYSTEM *system = al_get_system_driver();
-   return system->vt->get_display_mode(index, mode);
-}
-
-
-
 /* Function: al_wait_for_vsync
  */
 bool al_wait_for_vsync(void)
@@ -344,11 +287,29 @@ bool al_wait_for_vsync(void)
  */
 void al_set_display_icon(ALLEGRO_DISPLAY *display, ALLEGRO_BITMAP *icon)
 {
-   ASSERT(display);
-   ASSERT(icon);
+   ALLEGRO_BITMAP *icons[1] = { icon };
 
-   if (display->vt->set_icon) {
-      display->vt->set_icon(display, icon);
+   al_set_display_icons(display, 1, icons);
+}
+
+
+
+/* Function: al_set_display_icons
+ */
+void al_set_display_icons(ALLEGRO_DISPLAY *display,
+   int num_icons, ALLEGRO_BITMAP *icons[])
+{
+   int i;
+
+   ASSERT(display);
+   ASSERT(num_icons >= 1);
+   ASSERT(icons);
+   for (i = 0; i < num_icons; i++) {
+      ASSERT(icons[i]);
+   }
+
+   if (display->vt->set_icons) {
+      display->vt->set_icons(display, num_icons, icons);
    }
 }
 
@@ -366,37 +327,6 @@ void _al_destroy_display_bitmaps(ALLEGRO_DISPLAY *d)
 }
 
 
-/* Function: al_get_num_video_adapters
- */
-int al_get_num_video_adapters(void)
-{
-   ALLEGRO_SYSTEM *system = al_get_system_driver();
-
-   if (system && system->vt && system->vt->get_num_video_adapters) {
-      return system->vt->get_num_video_adapters();
-   }
-
-   return 0;
-}
-
-
-/* Function: al_get_monitor_info
- */
-bool al_get_monitor_info(int adapter, ALLEGRO_MONITOR_INFO *info)
-{
-   ALLEGRO_SYSTEM *system = al_get_system_driver();
-
-   if (adapter < al_get_num_video_adapters()) {
-      if (system && system->vt && system->vt->get_monitor_info) {
-         return system->vt->get_monitor_info(adapter, info);
-      }
-   }
-
-   info->x1 = info->y1 = info->x2 = info->y2 = INT_MAX;
-   return false;
-}
-
-
 /* Function: al_set_window_position
  */
 void al_set_window_position(ALLEGRO_DISPLAY *display, int x, int y)
