@@ -647,14 +647,16 @@ static unsigned int mouse_get_mouse_num_axes(void)
 /* mouse_set_mouse_xy:
  *
  */
-static bool mouse_set_mouse_xy(int x, int y)
+static bool mouse_set_mouse_xy(ALLEGRO_DISPLAY *display, int x, int y)
 {
+   (void)display;
+
    _al_event_source_lock(&the_mouse.parent.es);
    {
       int dx, dy;
 
-      x_axis.out_abs = CLAMP(x_axis.out_min, x_axis.out_abs, x_axis.out_max);
-      y_axis.out_abs = CLAMP(y_axis.out_min, y_axis.out_abs, y_axis.out_max);
+      x_axis.out_abs = _ALLEGRO_CLAMP(x_axis.out_min, x, x_axis.out_max);
+      y_axis.out_abs = _ALLEGRO_CLAMP(y_axis.out_min, y, y_axis.out_max);
       x_axis.mickeys = 0;
       y_axis.mickeys = 0;
 
