@@ -1,3 +1,5 @@
+#error This driver has not been updated to the new mouse API.
+
 /*         ______   ___    ___
  *        /\  _  \ /\_ \  /\_ \
  *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
@@ -22,10 +24,10 @@
 #include <sys/time.h>
 #include <termios.h>
 
-#include "allegro.h"
-#include "allegro/internal/aintern.h"
-#include "allegro/platform/aintunix.h"
-#include "linalleg.h"
+#include "allegro5/allegro.h"
+#include "allegro5/internal/aintern.h"
+#include "allegro5/platform/aintunix.h"
+#include "allegro5/linalleg.h"
 
 
 static int intellimouse;
@@ -80,7 +82,7 @@ static int processor (unsigned char *buf, int buf_size)
  *  if there's a reasonable chance that this driver can work with that
  *  data.
  */
-static int analyse_data (AL_CONST char *buffer, int size)
+static int analyse_data (const char *buffer, int size)
 {
 	int pos = 0;
 	int packets = 0, errors = 0;
@@ -149,7 +151,7 @@ static void sync_mouse (int fd)
 static int mouse_init (void)
 {
 	char tmp1[128], tmp2[128], tmp3[128];
-	AL_CONST char *udevice;
+	const char *udevice;
 	struct termios t;
 
 	/* Find the device filename */
@@ -183,7 +185,7 @@ static int mouse_init (void)
  */
 static int ms_mouse_init (void)
 {
-	intellimouse = FALSE;
+	intellimouse = false;
     	packet_size = 3;
 	intdrv.num_buttons = 2;
 	return mouse_init ();
@@ -194,7 +196,7 @@ static int ms_mouse_init (void)
  */
 static int ims_mouse_init (void)
 {
-	intellimouse = TRUE;
+	intellimouse = true;
     	packet_size = 4;
 	intdrv.num_buttons = 3;
 	return mouse_init ();
@@ -212,8 +214,8 @@ static void mouse_exit (void)
 MOUSE_DRIVER mousedrv_linux_ms =
 {
 	MOUSEDRV_LINUX_MS,
-	empty_string,
-	empty_string,
+	"",
+	"",
 	"Linux MS mouse",
 	ms_mouse_init,
 	mouse_exit,
@@ -231,8 +233,8 @@ MOUSE_DRIVER mousedrv_linux_ms =
 MOUSE_DRIVER mousedrv_linux_ims =
 {
 	MOUSEDRV_LINUX_IMS,
-	empty_string,
-	empty_string,
+	"",
+	"",
 	"Linux MS Intellimouse",
 	ims_mouse_init,
 	mouse_exit,
