@@ -1,3 +1,5 @@
+#error This file is no longer used.
+
 /*         ______   ___    ___ 
  *        /\  _  \ /\_ \  /\_ \ 
  *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
@@ -19,10 +21,10 @@
 #define _GNU_SOURCE
 #endif
 
-#include "allegro.h"
-#include "allegro/internal/aintern.h"
-#include "allegro/platform/aintunix.h"
-#include "linalleg.h"
+#include "allegro5/allegro.h"
+#include "allegro5/internal/aintern.h"
+#include "allegro5/platform/aintunix.h"
+#include "allegro5/linalleg.h"
 
 #include <unistd.h>
 
@@ -30,7 +32,7 @@
 /* List of standard drivers */
 STD_DRIVER *__al_linux_std_drivers[N_STD_DRIVERS];
 
-static int std_drivers_suspended = FALSE;
+static int std_drivers_suspended = false;
 
 
 /* __al_linux_add_standard_driver:
@@ -90,7 +92,7 @@ void __al_linux_update_standard_drivers (int threaded)
 void __al_linux_suspend_standard_drivers (void)
 {
    ASSERT(!std_drivers_suspended);
-   std_drivers_suspended = TRUE;
+   std_drivers_suspended = true;
 }
 
 /* __al_linux_resume_standard_drivers:
@@ -99,6 +101,6 @@ void __al_linux_suspend_standard_drivers (void)
 void __al_linux_resume_standard_drivers (void)
 {
    ASSERT(std_drivers_suspended);
-   std_drivers_suspended = FALSE;
+   std_drivers_suspended = false;
 }
 
