@@ -25,16 +25,10 @@
 
 #include "allegro5/allegro.h"
 #include "allegro5/internal/aintern.h"
+#include "allegro5/internal/aintern_driver.h"
 #include "allegro5/platform/aintunix.h"
 #include "allegro5/platform/aintlnx.h"
 
-#ifdef ALLEGRO_LINUX_VGA
-#ifdef ALLEGRO_HAVE_SYS_IO_H
-/* iopl() exists in here instead of unistd.h in glibc */
-#include <sys/io.h>
-#endif
-#endif
-
 #include "allegro5/linalleg.h"
 
 #ifndef ALLEGRO_LINUX
@@ -88,13 +82,8 @@ SYSTEM_DRIVER system_linux =
    NULL, /* get_vtable */
    __al_linux_set_display_switch_mode,
    __al_linux_display_switch_lock,
-#if (defined ALLEGRO_LINUX_FBCON) && (!defined ALLEGRO_WITH_MODULES)
-   __al_linux_get_fb_color_depth,
-   __al_linux_get_fb_resolution,
-#else
    NULL, /* desktop_color_depth */
    NULL, /* get_desktop_resolution */
-#endif
    NULL, /* get_gfx_safe_mode */
    _unix_yield_timeslice,
    get_gfx_driver_list,
@@ -109,14 +98,14 @@ SYSTEM_DRIVER system_linux =
 int __al_linux_have_ioperms = 0;
 
 
-typedef RETSIGTYPE (*temp_sighandler_t)(int);
+typedef void (*temp_sighandler_t)(int);
 static temp_sighandler_t old_sig_abrt, old_sig_fpe, old_sig_ill, old_sig_segv, old_sig_term, old_sig_int, old_sig_quit;
 
 
 /* signal_handler:
  *  Used to trap various signals, to make sure things get shut down cleanly.
  */
-static RETSIGTYPE signal_handler (int num)
+static void signal_handler (int num)
 {
 	al_uninstall_system();
 	fprintf (stderr, "Shutting down Allegro due to signal #%d\n", num);
@@ -163,28 +152,11 @@ static int sys_linux_init (void)
 	 * we attempt to set our euid to 0, in case this is the
 	 * second time we've been called. */
 	__al_linux_have_ioperms  = !seteuid (0);
-#ifdef ALLEGRO_LINUX_VGA
-	__al_linux_have_ioperms &= !iopl (3);
-#endif
 	__al_linux_have_ioperms &= !__al_linux_init_memory();
 
 	/* At this stage we can drop the root privileges. */
 	seteuid (getuid());
 
-	/* Initialise dynamic driver lists */
-	_unix_driver_lists_init();
-	if (_unix_gfx_driver_list)
-		_driver_list_append_list(&_unix_gfx_driver_list, _linux_gfx_driver_list);
-
-	/* Load dynamic modules */
-	_unix_load_modules(SYSTEM_LINUX);
-    
-	/* Initialise VGA helpers */
-#ifdef ALLEGRO_LINUX_VGA
-	if (__al_linux_have_ioperms)
-		if (__al_linux_init_vga_helpers()) return -1;
-#endif
-
 	/* Install emergency-exit signal handlers */
 	old_sig_abrt = signal(SIGABRT, signal_handler);
 	old_sig_fpe  = signal(SIGFPE,  signal_handler);
@@ -230,22 +202,7 @@ static void sys_linux_exit (void)
 	signal(SIGQUIT, old_sig_quit);
 #endif
 
-	/* shut down VGA helpers */
-#ifdef ALLEGRO_LINUX_VGA
- 	if (__al_linux_have_ioperms)
-		__al_linux_shutdown_vga_helpers();
-#endif
-
-	/* unload dynamic modules */
-	_unix_unload_modules();
-
-	/* free dynamic driver lists */
-	_unix_driver_lists_shutdown();
-
 	__al_linux_shutdown_memory();
-#ifdef ALLEGRO_LINUX_VGA
-	iopl (0);
-#endif
 }
 
 
