@@ -35,6 +35,16 @@
 
 ALLEGRO_DEBUG_CHANNEL("MacOSX")
 
+/* This constant isn't available on OS X < 10.7, define
+ * it here so the library can be built on < 10.7 (10.6
+ * tested so far.)
+ */
+#if (MAC_OS_X_VERSION_MAX_ALLOWED < 1070 && !defined(NSWindowCollectionBehaviorFullScreenPrimary))
+enum {
+   NSWindowCollectionBehaviorFullScreenPrimary = (1 << 7)
+};
+#endif
+
 /* Defines */
 #define MINIMUM_WIDTH 48
 #define MINIMUM_HEIGHT 48
@@ -139,6 +149,16 @@ ALLEGRO_DISPLAY_INTERFACE* _al_osx_get_display_driver_fs(void);
 static NSOpenGLContext* osx_create_shareable_context(NSOpenGLPixelFormat* fmt, unsigned int* group);
 static bool set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff);
 
+static NSTrackingArea *create_tracking_area(NSView *view)
+{
+   NSTrackingAreaOptions options =
+      NSTrackingMouseEnteredAndExited |
+      NSTrackingMouseMoved |
+      NSTrackingCursorUpdate |
+      NSTrackingActiveAlways;
+   return [[NSTrackingArea alloc] initWithRect:[view bounds] options:options owner:view userInfo:nil];
+}
+
 /* osx_change_cursor:
  * Actually change the current cursor. This can be called fom any thread 
  * but ensures that the change is only called from the main thread.
@@ -197,6 +217,9 @@ void _al_osx_keyboard_was_installed(BOOL install) {
 -(void) windowDidBecomeMain:(NSNotification*) notification;
 -(void) windowDidResignMain:(NSNotification*) notification;
 -(void) windowDidResize:(NSNotification*) notification;
+-(void) enterFullScreenWindowMode;
+-(void) exitFullScreenWindowMode;
+-(void) finishExitingFullScreenWindowMode;
 @end
 
 /* ALWindow:
@@ -238,7 +261,7 @@ void _al_osx_keyboard_was_installed(BOOL install) {
 /* _al_osx_mouse_was_installed:
  * Called by the mouse driver when the driver is installed or uninstalled.
  * Set the variable so we can decide to pass events or not, and notify all
- * existing displays that they need to set up their tracking rectangles.
+ * existing displays that they need to set up their tracking areas.
  */
 void _al_osx_mouse_was_installed(BOOL install) {
    unsigned int i;
@@ -285,9 +308,10 @@ void _al_osx_mouse_was_installed(BOOL install) {
       glLoadIdentity();
       glOrtho(0, NSWidth(rc), NSHeight(rc), 0, -1, 1);
       
-      if ( dpy->tracking ) {
-         [self removeTrackingRect:dpy->tracking];
-         dpy->tracking = [self addTrackingRect:[self bounds] owner:self userData:nil assumeInside:NO];
+      if (dpy->tracking) {
+         [self removeTrackingArea: dpy->tracking];
+         dpy->tracking = create_tracking_area(self);
+	 [self addTrackingArea: dpy->tracking];
       }
    }
 }
@@ -381,14 +405,16 @@ void _al_osx_mouse_was_installed(BOOL install) {
 /* Cursor handling */
 - (void) viewDidMoveToWindow {
    ALLEGRO_DISPLAY_OSX_WIN* dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
-   dpy->tracking = [self addTrackingRect:[self bounds] owner:self userData:nil assumeInside:NO];
+   dpy->tracking = create_tracking_area(self);
+   [self addTrackingArea: dpy->tracking];
 }
 
 - (void) viewWillMoveToWindow: (NSWindow*) newWindow {
    ALLEGRO_DISPLAY_OSX_WIN* dpy = (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
    (void)newWindow;
    if (([self window] != nil) && (dpy->tracking != 0)) {
-      [self removeTrackingRect:dpy->tracking];
+      [self removeTrackingArea:dpy->tracking];
+      dpy->tracking = 0;
    }
 }
 
@@ -501,6 +527,41 @@ void _al_osx_mouse_was_installed(BOOL install) {
    }
    _al_event_source_unlock(es);
 }
+
+-(void) enterFullScreenWindowMode
+{
+#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 1060
+   ALLEGRO_DISPLAY_OSX_WIN *dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
+   NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
+   int flags = NSApplicationPresentationHideDock | NSApplicationPresentationHideMenuBar;
+   [dict setObject:[NSNumber numberWithInt: flags] forKey:NSFullScreenModeApplicationPresentationOptions];
+   [[dpy->win contentView] enterFullScreenMode: [dpy->win screen] withOptions: dict];
+   [dict release];
+#endif
+}
+
+-(void) exitFullScreenWindowMode
+{
+   ALLEGRO_DISPLAY_OSX_WIN *dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
+
+   /* Going from a fullscreen window to a smaller window, the mouse may end up outside
+    * the window when it was inside before (not possible the other way.) This causes a
+    * crash. To avoid it, remove the tracking area and add it back after exiting fullscreen.
+    * (my theory)
+    */
+   [dpy->win orderOut:[dpy->win contentView]];
+   [self exitFullScreenModeWithOptions: nil];
+}
+
+-(void) finishExitingFullScreenWindowMode
+{
+   ALLEGRO_DISPLAY_OSX_WIN *dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
+
+   [dpy->win center];
+   [dpy->win makeKeyAndOrderFront:[dpy->win contentView]];
+   [[self window] makeFirstResponder: self];
+}
+
 /* End of ALOpenGLView implementation */
 @end
 
@@ -822,6 +883,11 @@ static void osx_get_opengl_pixelformat_attributes(ALLEGRO_DISPLAY_OSX_WIN *dpy)
                 backing: NSBackingStoreBuffered
                   defer: NO
                  screen: screen];
+   if (dpy->parent.flags & ALLEGRO_RESIZABLE) {
+      if ([win respondsToSelector:NSSelectorFromString(@"setCollectionBehavior:")]) {
+         [win setCollectionBehavior:NSWindowCollectionBehaviorFullScreenPrimary];
+      }
+   }
    NSOpenGLPixelFormat* fmt =
       [[NSOpenGLPixelFormat alloc] initWithAttributes: dpy->attributes];
    ALOpenGLView* view = [[ALOpenGLView alloc] initWithFrame: rc];
@@ -886,7 +952,13 @@ static void osx_get_opengl_pixelformat_attributes(ALLEGRO_DISPLAY_OSX_WIN *dpy)
 }
 +(void) destroyDisplay: (NSValue*) display_object {
    ALLEGRO_DISPLAY_OSX_WIN* dpy = [display_object pointerValue];
+   ALLEGRO_OGL_EXTRAS *ogl = ((ALLEGRO_DISPLAY *)dpy)->ogl_extras;
    _al_vector_find_and_delete(&al_get_system_driver()->displays, &dpy);
+   if (ogl->backbuffer) {
+      _al_ogl_destroy_backbuffer(ogl->backbuffer);
+      ogl->backbuffer = NULL;
+      ALLEGRO_DEBUG("destroy backbuffer.\n");
+   }
    // Disconnect from its view or exit fullscreen mode
    [dpy->ctx clearDrawable];
    // Unlock the screen 
@@ -1386,8 +1458,11 @@ static ALLEGRO_DISPLAY* create_display_win(int w, int h) {
    ALLEGRO_INFO("Vendor: %s\n", glGetString(GL_VENDOR));
    ALLEGRO_INFO("Renderer: %s\n", glGetString(GL_RENDERER));
    
-   // Set up a pixel format to describe the mode we want.
+   /* Set up a pixel format to describe the mode we want. */
    osx_set_opengl_pixelformat_attributes(dpy);
+   /* Retrieve the options that were set */
+   // Note: This initializes dpy->extra_settings
+   osx_get_opengl_pixelformat_attributes(dpy);
 
    // Set up the Allegro OpenGL implementation
    dpy->parent.ogl_extras = al_malloc(sizeof(ALLEGRO_OGL_EXTRAS));
@@ -1396,8 +1471,6 @@ static ALLEGRO_DISPLAY* create_display_win(int w, int h) {
    _al_ogl_set_extensions(dpy->parent.ogl_extras->extension_api);
    dpy->parent.ogl_extras->is_shared = true;
 
-   /* Retrieve the options that were set */
-   osx_get_opengl_pixelformat_attributes(dpy);
    dpy->parent.ogl_extras->backbuffer = _al_ogl_create_backbuffer(&dpy->parent);
    if (_al_get_new_display_settings()->settings[ALLEGRO_VSYNC] == 1) {
       GLint swapInterval = 1;
@@ -1515,13 +1588,22 @@ static ALLEGRO_DISPLAY* create_display(int w, int h)
 static void flip_display(ALLEGRO_DISPLAY *disp) 
 {
    ALLEGRO_DISPLAY_OSX_WIN* dpy = (ALLEGRO_DISPLAY_OSX_WIN*) disp;
-   if (disp->ogl_extras->opengl_target->is_backbuffer) {
-      if (disp->extra_settings.settings[ALLEGRO_SINGLE_BUFFER]) {
-         glFlush();
-      }
-      else {
-         [dpy->ctx flushBuffer];
-      }
+
+   ALLEGRO_BITMAP *old_target = NULL;
+   if (!disp->ogl_extras->opengl_target->is_backbuffer) {
+   	old_target = al_get_target_bitmap();
+	al_set_target_backbuffer(disp);
+   }
+
+   if (disp->extra_settings.settings[ALLEGRO_SINGLE_BUFFER]) {
+      glFlush();
+   }
+   else {
+      [dpy->ctx flushBuffer];
+   }
+
+   if (old_target) {
+      al_set_target_bitmap(old_target);
    }
 }
 
@@ -1847,12 +1929,14 @@ static void set_window_title(ALLEGRO_DISPLAY *display, const char *title)
    [pool drain];
 }
 
-/* set_icon:
+/* set_icons:
  * Set the icon - OS X doesn't have per-window icons so 
  * ignore the display parameter
  */
-static void set_icon(ALLEGRO_DISPLAY *display, ALLEGRO_BITMAP* bitmap)
+static void set_icons(ALLEGRO_DISPLAY *display, int num_icons, ALLEGRO_BITMAP* bitmaps[])
 {
+   /* Multiple icons not yet implemented. */
+   ALLEGRO_BITMAP *bitmap = bitmaps[num_icons - 1];
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    NSImage *image = NSImageFromAllegroBitmap(bitmap);
    (void)display;
@@ -1900,24 +1984,22 @@ static bool set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
          [win setStyleMask : mask];
          return true;
 
-      case ALLEGRO_FULLSCREEN_WINDOW:
+      case ALLEGRO_FULLSCREEN_WINDOW: {
+         ALOpenGLView *view = (ALOpenGLView *)[win contentView];
          if (onoff) {
-            [[win contentView] enterFullScreenMode: [win screen] withOptions: nil];
+            [view performSelectorOnMainThread: @selector(enterFullScreenWindowMode) withObject:nil waitUntilDone:YES];
             NSRect sc = [[win screen] frame];
             resize_display_win(display, sc.size.width, sc.size.height);
             display->flags |= ALLEGRO_FULLSCREEN_WINDOW;
          } else {
-            [win orderOut:[win contentView]];
-            NSView *view = [win contentView];
-            [view exitFullScreenModeWithOptions: nil];
+            [view performSelectorOnMainThread: @selector(exitFullScreenWindowMode) withObject:nil waitUntilDone:YES];
             NSRect sc = [view frame];
             display->flags &= ~ALLEGRO_FULLSCREEN_WINDOW;
             resize_display_win(display, sc.size.width, sc.size.height);
-            [win center];
-            [win makeKeyAndOrderFront:[win contentView]];
-            [[view window] makeFirstResponder: view];
+            [view performSelectorOnMainThread: @selector(finishExitingFullScreenWindowMode) withObject:nil waitUntilDone:YES];
          }
          return true;
+      }
    }
 
    return false;
@@ -1950,7 +2032,7 @@ ALLEGRO_DISPLAY_INTERFACE* _al_osx_get_display_driver_win(void)
       vt->set_window_position = set_window_position;
       vt->set_window_title = set_window_title;
       vt->set_display_flag = set_display_flag;
-      vt->set_icon = set_icon;
+      vt->set_icons = set_icons;
       _al_ogl_add_drawing_functions(vt);
    }
    return vt;
