@@ -21,6 +21,7 @@
 #include "allegro5/allegro.h"
 #include "allegro5/allegro_opengl.h"
 #include "allegro5/internal/aintern.h"
+#include "allegro5/internal/aintern_driver.h"
 #include "allegro5/internal/aintern_mouse.h"
 #include "allegro5/internal/aintern_keyboard.h"
 #include "allegro5/platform/aintosx.h"
@@ -49,6 +50,7 @@ static struct {
 	ALLEGRO_MOUSE_STATE state;
 	float z_axis, w_axis;
 	BOOL warped;
+	int warped_x, warped_y;
 	NSCursor* cursor;
 } osx_mouse;
 
@@ -89,7 +91,7 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
 {
 	NSPoint pos;
 	int type, b_change = 0, dx = 0, dy = 0, dz = 0, dw = 0, b = 0;
-   float pressure = 0.0;
+        float pressure = 0.0;
 	switch ([evt type])
 	{
 		case NSMouseMoved:
@@ -113,6 +115,7 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
 			type = ALLEGRO_EVENT_MOUSE_BUTTON_DOWN;
 			b = [evt buttonNumber]+1;
 			b_change = 1;
+			osx_mouse.state.buttons |= (1 << (b-1));
          pressure = [evt pressure];
 			break;
 		case NSLeftMouseUp:
@@ -121,7 +124,8 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
 			type = ALLEGRO_EVENT_MOUSE_BUTTON_UP;
 			b = [evt buttonNumber]+1;
 			b_change = 1;
-         pressure = [evt pressure];
+			osx_mouse.state.buttons &= ~(1 << (b-1));
+                        pressure = [evt pressure];
 			break;
 		case NSScrollWheel:
 			type = ALLEGRO_EVENT_MOUSE_AXES;
@@ -161,9 +165,9 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
       pos.y = [[NSScreen mainScreen] frame].size.height - pos.y;
 	}
 	if (osx_mouse.warped) {
-	    dx = 0;
-	    dy = 0;
-	    osx_mouse.warped = FALSE;
+           dx = -osx_mouse.warped_x;
+           dy = -osx_mouse.warped_y;
+           osx_mouse.warped = FALSE;
 	}
 	_al_event_source_lock(&osx_mouse.parent.es);
 	if ((within || b_change || type == ALLEGRO_EVENT_MOUSE_LEAVE_DISPLAY)
@@ -175,7 +179,7 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
 		// Note: we use 'allegro time' rather than the time stamp 
 		// from the event 
 		mouse_event->timestamp = al_get_time();
-        mouse_event->display = dpy;
+                mouse_event->display = dpy;
 		mouse_event->button = b;
 		mouse_event->x = pos.x;
 		mouse_event->y = pos.y; 
@@ -185,7 +189,7 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
 		mouse_event->dy = dy;
 		mouse_event->dz = dz;
 		mouse_event->dw = dw;
-      mouse_event->pressure = pressure;
+                mouse_event->pressure = pressure;
 		_al_event_source_emit_event(&osx_mouse.parent.es, &new_event);
 	}
 	// Record current state
@@ -193,9 +197,7 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
 	osx_mouse.state.y = pos.y;
 	osx_mouse.state.w = osx_mouse.w_axis;
 	osx_mouse.state.z = osx_mouse.z_axis;
-   osx_mouse.state.pressure = pressure;
-	if (b_change)
-		osx_mouse.state.buttons ^= (1<<(b-1));
+        osx_mouse.state.pressure = pressure;
 	_al_event_source_unlock(&osx_mouse.parent.es);
 }
 
@@ -349,8 +351,10 @@ static bool osx_set_mouse_xy(ALLEGRO_DISPLAY *dpy_, int x, int y)
         mouse_event->dz = 0;
         mouse_event->pressure = 0.0;
         if (mouse_event->dx || mouse_event->dy) {
-            osx_mouse.warped = TRUE;
-            _al_event_source_emit_event(&osx_mouse.parent.es, &new_event);
+           osx_mouse.warped = TRUE;
+           osx_mouse.warped_x = mouse_event->dx;
+           osx_mouse.warped_y = mouse_event->dy;
+           _al_event_source_emit_event(&osx_mouse.parent.es, &new_event);
         }
     }
     CGDisplayMoveCursorToPoint(display, pos);
