@@ -20,10 +20,12 @@
 #include "allegro5/allegro.h"
 #include "allegro5/allegro_opengl.h"
 #include "allegro5/internal/aintern.h"
+#include "allegro5/internal/aintern_display.h"
+#include "allegro5/internal/aintern_memblit.h"
 #include "allegro5/internal/aintern_opengl.h"
 #include "allegro5/internal/aintern_pixels.h"
-#include "allegro5/internal/aintern_display.h"
 #include "allegro5/internal/aintern_system.h"
+#include "allegro5/internal/aintern_transform.h"
 #include <math.h>
 
 #if defined ALLEGRO_GP2XWIZ
@@ -169,58 +171,6 @@ static char const *error_string(GLenum e)
 }
 #undef ERR
 
-static INLINE bool setup_blending(ALLEGRO_DISPLAY *ogl_disp)
-{
-   int op, src_color, dst_color, op_alpha, src_alpha, dst_alpha;
-   const int blend_modes[4] = {
-      GL_ZERO, GL_ONE, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA
-   };
-   const int blend_equations[3] = {
-      GL_FUNC_ADD, GL_FUNC_SUBTRACT, GL_FUNC_REVERSE_SUBTRACT
-   };
-
-   (void)ogl_disp;
-
-   al_get_separate_blender(&op, &src_color, &dst_color,
-      &op_alpha, &src_alpha, &dst_alpha);
-   /* glBlendFuncSeparate was only included with OpenGL 1.4 */
-   /* (And not in OpenGL ES) */
-#if !defined ALLEGRO_GP2XWIZ
-#ifndef ALLEGRO_IPHONE
-   if (ogl_disp->ogl_extras->ogl_info.version >= _ALLEGRO_OPENGL_VERSION_1_4) {
-#else
-   if (ogl_disp->ogl_extras->ogl_info.version >= _ALLEGRO_OPENGL_VERSION_2_0) {
-#endif
-      glEnable(GL_BLEND);
-      glBlendFuncSeparate(blend_modes[src_color], blend_modes[dst_color],
-         blend_modes[src_alpha], blend_modes[dst_alpha]);
-      if (ogl_disp->ogl_extras->ogl_info.version >= _ALLEGRO_OPENGL_VERSION_2_0) {
-         glBlendEquationSeparate(
-            blend_equations[op],
-            blend_equations[op_alpha]);
-      }
-      else {
-         glBlendEquation(blend_equations[op]);
-      }
-   }
-   else {
-      if (src_color == src_alpha && dst_color == dst_alpha) {
-         glEnable(GL_BLEND);
-         glBlendFunc(blend_modes[src_color], blend_modes[dst_color]);
-      }
-      else {
-         ALLEGRO_ERROR("Blender unsupported with this OpenGL version (%d %d %d %d %d %d)\n",
-            op, src_color, dst_color, op_alpha, src_alpha, dst_alpha);
-         return false;
-      }
-   }
-#else
-   glEnable(GL_BLEND);
-   glBlendFunc(blend_modes[src_color], blend_modes[dst_color]);
-#endif
-   return true;
-}
-
 static INLINE void transform_vertex(float* x, float* y)
 {
    al_transform_coordinates(al_get_current_transform(), x, y);
@@ -330,7 +280,8 @@ static void ogl_draw_bitmap_region(ALLEGRO_BITMAP *bitmap,
 
    ogl_target = (ALLEGRO_BITMAP_OGL *)target;
 
-   if (!(bitmap->flags & ALLEGRO_MEMORY_BITMAP)) {
+   if (!(bitmap->flags & ALLEGRO_MEMORY_BITMAP) && !bitmap->locked &&
+         !target->locked) {
 #if !defined ALLEGRO_GP2XWIZ
       ALLEGRO_BITMAP_OGL *ogl_source = (void *)bitmap;
       if (ogl_source->is_backbuffer) {
@@ -396,7 +347,7 @@ static void ogl_draw_bitmap_region(ALLEGRO_BITMAP *bitmap,
 #endif
    }
    if (disp->ogl_extras->opengl_target == ogl_target) {
-      if (setup_blending(disp)) {
+      if (_al_opengl_set_blender(disp)) {
          draw_quad(bitmap, tint, sx, sy, sw, sh, flags);
          return;
       }
@@ -546,7 +497,7 @@ static bool ogl_upload_bitmap(ALLEGRO_BITMAP *bitmap)
 
    if (e) {
       ALLEGRO_ERROR("glTexImage2D for format %s, size %dx%d failed (%s)\n",
-         _al_format_name(bitmap->format),
+         _al_pixel_format_name(bitmap->format),
          ogl_bitmap->true_w, ogl_bitmap->true_h,
          error_string(e));
       glDeleteTextures(1, &ogl_bitmap->texture);
@@ -707,7 +658,7 @@ static ALLEGRO_LOCKED_REGION *ogl_lock_region(ALLEGRO_BITMAP *bitmap,
          e = glGetError();
          if (e) {
             ALLEGRO_ERROR("glReadPixels for format %s failed (%s).\n",
-               _al_format_name(format), error_string(e));
+               _al_pixel_format_name(format), error_string(e));
          }
       }
       bitmap->locked_region.data = ogl_bitmap->lock_buffer +
@@ -761,7 +712,7 @@ static ALLEGRO_LOCKED_REGION *ogl_lock_region(ALLEGRO_BITMAP *bitmap,
             e = glGetError();
             if (e) {
                ALLEGRO_ERROR("glReadPixels for format %s failed (%s).\n",
-                  _al_format_name(format), error_string(e));
+                  _al_pixel_format_name(format), error_string(e));
             }
 
             _al_convert_bitmap_data(
@@ -794,7 +745,7 @@ static ALLEGRO_LOCKED_REGION *ogl_lock_region(ALLEGRO_BITMAP *bitmap,
          e = glGetError();
          if (e) {
             ALLEGRO_ERROR("glGetTexImage for format %s failed (%s).\n",
-               _al_format_name(format), error_string(e));
+               _al_pixel_format_name(format), error_string(e));
          }
 
          bitmap->locked_region.data = ogl_bitmap->lock_buffer +
@@ -922,7 +873,7 @@ static void ogl_unlock_region(ALLEGRO_BITMAP *bitmap)
       e = glGetError();
       if (e) {
          ALLEGRO_ERROR("glDrawPixels for format %s failed (%s).\n",
-            _al_format_name(lock_format), error_string(e));
+            _al_pixel_format_name(lock_format), error_string(e));
       }
       if (popmatrix) {
          glPopMatrix();
@@ -951,7 +902,7 @@ static void ogl_unlock_region(ALLEGRO_BITMAP *bitmap)
       e = glGetError();
       if (e) {
          ALLEGRO_ERROR("glTexSubImage2D for format %s failed (%s).\n",
-            _al_format_name(lock_format), error_string(e));
+            _al_pixel_format_name(lock_format), error_string(e));
       }
 
       if (bitmap->flags & ALLEGRO_MIPMAP) {
@@ -1031,7 +982,7 @@ static void ogl_unlock_region(ALLEGRO_BITMAP *bitmap)
          if (e) {
             GLint tex_internalformat;
             ALLEGRO_ERROR("glTexSubImage2D for format %s failed (%s).\n",
-               _al_format_name(lock_format), error_string(e));
+               _al_pixel_format_name(lock_format), error_string(e));
 #ifndef ALLEGRO_IPHONE
             glGetTexLevelParameteriv(GL_TEXTURE_2D, 0,
                GL_TEXTURE_INTERNAL_FORMAT, &tex_internalformat);
@@ -1157,7 +1108,7 @@ ALLEGRO_BITMAP *_al_ogl_create_bitmap(ALLEGRO_DISPLAY *d, int w, int h)
       format = ALLEGRO_PIXEL_FORMAT_RGBA_4444;
 #endif
 
-   ALLEGRO_DEBUG("Chose format %s for OpenGL bitmap\n", _al_format_name(format));
+   ALLEGRO_DEBUG("Chose format %s for OpenGL bitmap\n", _al_pixel_format_name(format));
    ASSERT(_al_pixel_format_is_real(format));
 
    pitch = true_w * al_get_pixel_size(format);
@@ -1327,6 +1278,10 @@ void al_get_opengl_texture_size(ALLEGRO_BITMAP *bitmap, int *w, int *h)
  */
 void al_get_opengl_texture_position(ALLEGRO_BITMAP *bitmap, int *u, int *v)
 {
+   ASSERT(bitmap);
+   ASSERT(u);
+   ASSERT(v);
+
    *u = bitmap->xofs;
    *v = bitmap->yofs;
 }
