@@ -27,6 +27,7 @@
 #include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_bitmap.h"
 #include "allegro5/internal/aintern_display.h"
+#include "allegro5/internal/aintern_pixels.h"
 #include "allegro5/internal/aintern_system.h"
 #include "allegro5/internal/aintern_thread.h"
 #include "allegro5/internal/aintern_vector.h"
@@ -234,7 +235,7 @@ bool al_have_d3d_non_square_texture_support(void)
 }
 
 
-int _al_format_to_d3d(int format)
+int _al_pixel_format_to_d3d(int format)
 {
    int i;
 
@@ -389,7 +390,7 @@ static bool d3d_display_mode_matches(D3DDISPLAYMODE *dm, int w, int h, int forma
    if ((dm->Width == (unsigned int)w) &&
        (dm->Height == (unsigned int)h) &&
        ((!refresh_rate) || (dm->RefreshRate == (unsigned int)refresh_rate)) &&
-       ((int)dm->Format == (int)_al_format_to_d3d(format))) {
+       ((int)dm->Format == (int)_al_pixel_format_to_d3d(format))) {
           return true;
    }
    return false;
@@ -401,10 +402,10 @@ static bool d3d_check_mode(int w, int h, int format, int refresh_rate, UINT adap
    UINT i;
    D3DDISPLAYMODE display_mode;
 
-   num_modes = _al_d3d->GetAdapterModeCount(adapter, (D3DFORMAT)_al_format_to_d3d(format));
+   num_modes = _al_d3d->GetAdapterModeCount(adapter, (D3DFORMAT)_al_pixel_format_to_d3d(format));
 
    for (i = 0; i < num_modes; i++) {
-      if (_al_d3d->EnumAdapterModes(adapter, (D3DFORMAT)_al_format_to_d3d(format), i, &display_mode) != D3D_OK) {
+      if (_al_d3d->EnumAdapterModes(adapter, (D3DFORMAT)_al_pixel_format_to_d3d(format), i, &display_mode) != D3D_OK) {
          return false;
       }
       if (d3d_display_mode_matches(&display_mode, w, h, format, refresh_rate)) {
@@ -439,7 +440,7 @@ static bool d3d_create_fullscreen_device(ALLEGRO_DISPLAY_D3D *d,
    }
 
    ZeroMemory(&d3d_pp, sizeof(d3d_pp));
-   d3d_pp.BackBufferFormat = (D3DFORMAT)_al_format_to_d3d(format);
+   d3d_pp.BackBufferFormat = (D3DFORMAT)_al_pixel_format_to_d3d(format);
    d3d_pp.BackBufferWidth = al_display->w;
    d3d_pp.BackBufferHeight = al_display->h;
    d3d_pp.BackBufferCount = 1;
@@ -778,7 +779,7 @@ static bool d3d_create_device(ALLEGRO_DISPLAY_D3D *d,
 
    ZeroMemory(&d3d_pp, sizeof(d3d_pp));
 
-   d3d_pp.BackBufferFormat = (D3DFORMAT)_al_format_to_d3d(format);
+   d3d_pp.BackBufferFormat = (D3DFORMAT)_al_pixel_format_to_d3d(format);
 
    d3d_pp.BackBufferWidth = al_display->w;
    d3d_pp.BackBufferHeight = al_display->h;
@@ -973,7 +974,7 @@ static bool _al_d3d_reset_device(ALLEGRO_DISPLAY_D3D *d3d_display)
        HRESULT hr;
 
        ZeroMemory(&d3d_pp, sizeof(d3d_pp));
-       d3d_pp.BackBufferFormat = (D3DFORMAT)_al_format_to_d3d(_al_deduce_color_format(&al_display->extra_settings));
+       d3d_pp.BackBufferFormat = (D3DFORMAT)_al_pixel_format_to_d3d(_al_deduce_color_format(&al_display->extra_settings));
        d3d_pp.BackBufferWidth = al_display->w;
        d3d_pp.BackBufferHeight = al_display->h;
        d3d_pp.BackBufferCount = 1;
@@ -1052,7 +1053,7 @@ static bool _al_d3d_reset_device(ALLEGRO_DISPLAY_D3D *d3d_display)
     }
     else {
        ZeroMemory(&d3d_pp, sizeof(d3d_pp));
-       d3d_pp.BackBufferFormat = (D3DFORMAT)_al_format_to_d3d(_al_deduce_color_format(&al_display->extra_settings));
+       d3d_pp.BackBufferFormat = (D3DFORMAT)_al_pixel_format_to_d3d(_al_deduce_color_format(&al_display->extra_settings));
        d3d_pp.BackBufferWidth = al_display->w;
        d3d_pp.BackBufferHeight = al_display->h;
        d3d_pp.BackBufferCount = 1;
@@ -1169,7 +1170,7 @@ static int real_choose_bitmap_format(ALLEGRO_DISPLAY_D3D *d3d_display,
          ALLEGRO_DEBUG("#Bits don't match\n");
          continue;
       }
-      if (alpha && !_al_format_has_alpha(aformat)) {
+      if (alpha && !_al_pixel_format_has_alpha(aformat)) {
          ALLEGRO_DEBUG("Alpha doesn't match\n");
          continue;
       }
@@ -1178,7 +1179,7 @@ static int real_choose_bitmap_format(ALLEGRO_DISPLAY_D3D *d3d_display,
       if (!_al_pixel_format_is_real(adapter_format_allegro))
          adapter_format_allegro = d3d_choose_display_format(adapter_format_allegro);
       ALLEGRO_DEBUG("Adapter format is %d\n", adapter_format_allegro);
-      adapter_format = (D3DFORMAT)_al_format_to_d3d(adapter_format_allegro);
+      adapter_format = (D3DFORMAT)_al_pixel_format_to_d3d(adapter_format_allegro);
       if (IsTextureFormatOk(dformat, adapter_format)) {
          ALLEGRO_DEBUG("Found a format\n");
          return aformat;
@@ -1462,6 +1463,10 @@ static void *d3d_display_thread_proc(void *arg)
       }
       else if (hr == D3DERR_DEVICELOST) {
          /* device remains lost */
+         /* Set device_lost flag immediately.  This prevents a crash in
+          * the DrawPrimitiveUP call in d3d_flush_vertex_cache.
+          */
+         d3d_display->device_lost = true;
          if (!lost_event_generated) {
             ALLEGRO_DEBUG("D3DERR_DEVICELOST: d3d_display=%p\n", d3d_display);
             lost_event_generated = true;
@@ -1827,33 +1832,18 @@ static bool d3d_set_current_display(ALLEGRO_DISPLAY *d)
 
 static int d3d_al_blender_to_d3d(int al_mode)
 {
-   int num_modes = 4;
-
-   int allegro_modes[] = {
-      ALLEGRO_ZERO,
-      ALLEGRO_ONE,
-      ALLEGRO_ALPHA,
-      ALLEGRO_INVERSE_ALPHA
-   };
-
-   int d3d_modes[] = {
+   const int d3d_modes[ALLEGRO_NUM_BLEND_MODES] = {
       D3DBLEND_ZERO,
       D3DBLEND_ONE,
       D3DBLEND_SRCALPHA,
-      D3DBLEND_INVSRCALPHA
+      D3DBLEND_INVSRCALPHA,
+      D3DBLEND_SRCCOLOR,
+      D3DBLEND_DESTCOLOR,
+      D3DBLEND_INVSRCCOLOR,
+      D3DBLEND_INVDESTCOLOR
    };
 
-   int i;
-
-   for (i = 0; i < num_modes; i++) {
-      if (al_mode == allegro_modes[i]) {
-         return d3d_modes[i];
-      }
-   }
-
-   ALLEGRO_WARN("Unknown blend mode.\n");
-
-   return D3DBLEND_ONE;
+   return d3d_modes[al_mode];
 }
 
 void _al_d3d_set_blender(ALLEGRO_DISPLAY_D3D *d3d_display)
@@ -2234,11 +2224,14 @@ static bool d3d_resize_helper(ALLEGRO_DISPLAY *d, int width, int height)
 
 static bool d3d_resize_display(ALLEGRO_DISPLAY *d, int width, int height)
 {
+   ALLEGRO_DISPLAY_D3D *d3d_display = (ALLEGRO_DISPLAY_D3D *)d;
    ALLEGRO_DISPLAY_WIN *win_display = (ALLEGRO_DISPLAY_WIN *)d;
    int orig_w = d->w;
    int orig_h = d->h;
    bool ret;
 
+   _al_d3d_prepare_bitmaps_for_reset(d3d_display);
+
    win_display->ignore_resize = true;
 
    if (!d3d_resize_helper(d, width, height)) {
@@ -2269,6 +2262,7 @@ ALLEGRO_BITMAP *_al_d3d_create_bitmap(ALLEGRO_DISPLAY *d,
    ASSERT(bitmap);
    (void)h;
 
+   memset(bitmap, 0, sizeof(*bitmap));
    bitmap->bitmap.size = sizeof *bitmap;
 
    format = al_get_new_bitmap_format();
@@ -2281,7 +2275,7 @@ ALLEGRO_BITMAP *_al_d3d_create_bitmap(ALLEGRO_DISPLAY *d,
       }
    }
 
-   if (_al_format_to_d3d(format) < 0) {
+   if (_al_pixel_format_to_d3d(format) < 0) {
       ALLEGRO_ERROR("Requested bitmap format not supported (%d).\n", format);
       return NULL;
    }
@@ -2465,6 +2459,10 @@ LPDIRECT3DTEXTURE9 al_get_d3d_video_texture(ALLEGRO_BITMAP *bitmap)
  */
 void al_get_d3d_texture_position(ALLEGRO_BITMAP *bitmap, int *u, int *v)
 {
+   ASSERT(bitmap);
+   ASSERT(u);
+   ASSERT(v);
+
    *u = bitmap->xofs;
    *v = bitmap->yofs;
 }
@@ -2632,7 +2630,7 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_d3d_driver(void)
    vt->show_mouse_cursor = _al_win_show_mouse_cursor;
    vt->hide_mouse_cursor = _al_win_hide_mouse_cursor;
 
-   vt->set_icon = _al_win_set_display_icon;
+   vt->set_icons = _al_win_set_display_icons;
    vt->set_window_position = d3d_set_window_position;
    vt->get_window_position = d3d_get_window_position;
    vt->set_display_flag = _al_win_set_display_flag;
@@ -2675,7 +2673,7 @@ int _al_d3d_get_num_display_modes(int format, int refresh_rate, int flags)
       if (adapter < 0)
          adapter = 0;
 
-      if (!_al_pixel_format_is_real(allegro_formats[j]) || _al_format_has_alpha(allegro_formats[j]))
+      if (!_al_pixel_format_is_real(allegro_formats[j]) || _al_pixel_format_has_alpha(allegro_formats[j]))
          continue;
 
       num_modes = _al_d3d->GetAdapterModeCount(adapter, (D3DFORMAT)d3d_formats[j]);
@@ -2725,7 +2723,7 @@ ALLEGRO_DISPLAY_MODE *_al_d3d_get_display_mode(int index, int format,
       if (adapter < 0)
          adapter = 0;
 
-      if (!_al_pixel_format_is_real(allegro_formats[j]) || _al_format_has_alpha(allegro_formats[j]))
+      if (!_al_pixel_format_is_real(allegro_formats[j]) || _al_pixel_format_has_alpha(allegro_formats[j]))
          continue;
 
       num_modes = _al_d3d->GetAdapterModeCount(adapter, (D3DFORMAT)d3d_formats[j]);
