@@ -1,95 +1,23 @@
-#include "allegro5/internal/aintern_xglx.h"
+#include <X11/Xatom.h>
+
+#include "allegro5/allegro.h"
+#include "allegro5/allegro_opengl.h"
 #include "allegro5/internal/aintern_bitmap.h"
 #include "allegro5/internal/aintern_opengl.h"
-
+#include "allegro5/internal/aintern_x.h"
+#include "allegro5/internal/aintern_xcursor.h"
+#include "allegro5/internal/aintern_xdisplay.h"
+#include "allegro5/internal/aintern_xfullscreen.h"
+#include "allegro5/internal/aintern_xglx_config.h"
+#include "allegro5/internal/aintern_xsystem.h"
+#include "allegro5/internal/aintern_xwindow.h"
+#include "allegro5/platform/aintxglx.h"
 
 ALLEGRO_DEBUG_CHANNEL("display")
 
 static ALLEGRO_DISPLAY_INTERFACE xdpy_vt;
 
 
-
-/* Helper to set up GL state as we want it. */
-static void setup_gl(ALLEGRO_DISPLAY *d)
-{
-   ALLEGRO_OGL_EXTRAS *ogl = d->ogl_extras;
-
-   glViewport(0, 0, d->w, d->h);
-
-   glMatrixMode(GL_PROJECTION);
-   glLoadIdentity();
-   glOrtho(0, d->w, d->h, 0, -1, 1);
-
-   glMatrixMode(GL_MODELVIEW);
-   glLoadIdentity();
-
-   if (ogl->backbuffer)
-      _al_ogl_resize_backbuffer(ogl->backbuffer, d->w, d->h);
-   else
-      ogl->backbuffer = _al_ogl_create_backbuffer(d);
-}
-
-
-
-static void set_size_hints(ALLEGRO_DISPLAY *d, int x_off, int y_off)
-{
-   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
-   ALLEGRO_DISPLAY_XGLX *glx = (void *)d;
-   XSizeHints *hints = XAllocSizeHints();
-
-   /* Do not force the size of the window on resizeable or fullscreen windows */
-   /* on fullscreen windows, it confuses most X Window Managers */
-   if (!(d->flags & ALLEGRO_RESIZABLE) && !(d->flags & ALLEGRO_FULLSCREEN)) {
-      hints->flags = PMinSize | PMaxSize | PBaseSize;
-      hints->min_width  = hints->max_width  = hints->base_width  = d->w;
-      hints->min_height = hints->max_height = hints->base_height = d->h;
-   }
-
-   // Tell WMs to respect our chosen position, otherwise the x_off/y_off
-   // positions passed to XCreateWindow will be ignored by most WMs.
-   if (x_off != INT_MAX && y_off != INT_MAX) {
-      ALLEGRO_DEBUG("Force window position to %d, %d.\n", x_off, y_off);
-      hints->flags |= PPosition;
-      hints->x = x_off;
-      hints->y = y_off;
-   }
-
-   if (d->flags & ALLEGRO_FULLSCREEN) {
-      /* kwin will improperly layer a panel over our window on a second display without this.
-       * some other Size flags may cause glitches with various WMs, but this seems to be ok
-       * with metacity and kwin. As noted in xdpy_create_display, compiz is just broken.
-       */
-      hints->flags |= PBaseSize;
-      hints->base_width  = d->w;
-      hints->base_height = d->h;
-   }
-
-   XSetWMNormalHints(system->x11display, glx->window, hints);
-
-   XFree(hints);
-}
-
-
-
-static void reset_size_hints(ALLEGRO_DISPLAY *d)
-{
-   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
-   ALLEGRO_DISPLAY_XGLX *glx = (void *)d;
-   XSizeHints *hints = XAllocSizeHints();
-
-   hints->flags = PMinSize | PMaxSize;
-   hints->min_width  = 0;
-   hints->min_height = 0;
-   // FIXME: Is there a way to remove/reset max dimensions?
-   hints->max_width  = 32768;
-   hints->max_height = 32768;
-   XSetWMNormalHints(system->x11display, glx->window, hints);
-
-   XFree(hints);
-}
-
-
-
 /* Helper to set a window icon.  We use the _NET_WM_ICON property which is
  * supported by modern window managers.
  *
@@ -97,49 +25,74 @@ static void reset_size_hints(ALLEGRO_DISPLAY *d)
  * pixmaps.  For colour icons, perhaps you're supposed use the icon_window,
  * and draw the window yourself?
  */
-static void xdpy_set_icon(ALLEGRO_DISPLAY *d, ALLEGRO_BITMAP *bitmap)
+static bool xdpy_set_icon_inner(Display *x11display, Window window,
+   ALLEGRO_BITMAP *bitmap, int prop_mode)
+{
+   int w, h;
+   int data_size;
+   unsigned long *data; /* Yes, unsigned long, even on 64-bit platforms! */
+   ALLEGRO_LOCKED_REGION *lr;
+   bool ret;
+
+   w = al_get_bitmap_width(bitmap);
+   h = al_get_bitmap_height(bitmap);
+   data_size = 2 + w * h;
+   data = al_malloc(data_size * sizeof(data[0]));
+   if (!data)
+      return false;
+
+   lr = al_lock_bitmap(bitmap, ALLEGRO_PIXEL_FORMAT_ANY_WITH_ALPHA,
+      ALLEGRO_LOCK_READONLY);
+   if (lr) {
+      int x, y;
+      ALLEGRO_COLOR c;
+      unsigned char r, g, b, a;
+      Atom _NET_WM_ICON;
+
+      data[0] = w;
+      data[1] = h;
+      for (y = 0; y < h; y++) {
+         for (x = 0; x < w; x++) {
+            c = al_get_pixel(bitmap, x, y);
+            al_unmap_rgba(c, &r, &g, &b, &a);
+            data[2 + y*w + x] = (a << 24) | (r << 16) | (g << 8) | b;
+         }
+      }
+
+      _NET_WM_ICON = XInternAtom(x11display, "_NET_WM_ICON", False);
+      XChangeProperty(x11display, window, _NET_WM_ICON, XA_CARDINAL, 32,
+         prop_mode, (unsigned char *)data, data_size);
+
+      al_unlock_bitmap(bitmap);
+      ret = true;
+   }
+   else {
+      ret = false;
+   }
+
+   al_free(data);
+
+   return ret;
+}
+
+static void xdpy_set_icons(ALLEGRO_DISPLAY *d,
+   int num_icons, ALLEGRO_BITMAP *bitmaps[])
 {
    ALLEGRO_SYSTEM_XGLX *system = (ALLEGRO_SYSTEM_XGLX *)al_get_system_driver();
    ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)d;
-   int w = al_get_bitmap_width(bitmap);
-   int h = al_get_bitmap_height(bitmap);
-   int data_size;
-   unsigned long *data; /* Yes, unsigned long, even on 64-bit platforms! */
+   int prop_mode = PropModeReplace;
+   int i;
 
-   data_size = 2 + w * h;
-   data = al_malloc(data_size * sizeof(data[0]));
+   _al_mutex_lock(&system->lock);
 
-   if (data) {
-      ALLEGRO_LOCKED_REGION *lr = al_lock_bitmap(bitmap,
-         ALLEGRO_PIXEL_FORMAT_ANY_WITH_ALPHA, ALLEGRO_LOCK_READONLY);
-
-      if (lr) {
-         int x, y;
-         ALLEGRO_COLOR c;
-         unsigned char r, g, b, a;
-         Atom _NET_WM_ICON;
-
-         data[0] = w;
-         data[1] = h;
-         for (y = 0; y < h; y++) {
-            for (x = 0; x < w; x++) {
-               c = al_get_pixel(bitmap, x, y);
-               al_unmap_rgba(c, &r, &g, &b, &a);
-               data[2 + y*w + x] = (a << 24) | (r << 16) | (g << 8) | b;
-            }
-         }
-
-         _al_mutex_lock(&system->lock);
-         _NET_WM_ICON = XInternAtom(system->x11display, "_NET_WM_ICON", False);
-         XChangeProperty(system->x11display, glx->window, _NET_WM_ICON,
-            XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data, data_size);
-         _al_mutex_unlock(&system->lock);
-
-         al_unlock_bitmap(bitmap);
+   for (i = 0; i < num_icons; i++) {
+      if (xdpy_set_icon_inner(system->x11display, glx->window, bitmaps[i],
+            prop_mode)) {
+         prop_mode = PropModeAppend;
       }
-
-      al_free(data);
    }
+
+   _al_mutex_unlock(&system->lock);
 }
 
 
@@ -221,14 +174,14 @@ static void xdpy_set_fullscreen_window(ALLEGRO_DISPLAY *display, bool onoff)
    ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
    if (onoff == !(display->flags & ALLEGRO_FULLSCREEN_WINDOW)) {
       _al_mutex_lock(&system->lock);
-      reset_size_hints(display);
-      _al_xglx_set_fullscreen_window(display, 2);
+      _al_xwin_reset_size_hints(display);
+      _al_xwin_set_fullscreen_window(display, 2);
       /* XXX Technically, the user may fiddle with the _NET_WM_STATE_FULLSCREEN
        * property outside of Allegro so this flag may not be in sync with
        * reality.
        */
       display->flags ^= ALLEGRO_FULLSCREEN_WINDOW;
-      set_size_hints(display, INT_MAX, INT_MAX);
+      _al_xwin_set_size_hints(display, INT_MAX, INT_MAX);
       _al_mutex_unlock(&system->lock);
    }
 }
@@ -267,7 +220,6 @@ static void _al_xglx_unuse_adapter(ALLEGRO_SYSTEM_XGLX *s, int adapter)
 static void xdpy_destroy_display(ALLEGRO_DISPLAY *d);
 
 
-
 /* Create a new X11 display, which maps directly to a GLX window. */
 static ALLEGRO_DISPLAY *xdpy_create_display(int w, int h)
 {
@@ -298,7 +250,7 @@ static ALLEGRO_DISPLAY *xdpy_create_display(int w, int h)
 
    display->w = w;
    display->h = h;
-   display->vt = &xdpy_vt;
+   display->vt = _al_display_xglx_driver();
    display->refresh_rate = al_get_new_display_refresh_rate();
    display->flags = al_get_new_display_flags();
 
@@ -465,7 +417,7 @@ static ALLEGRO_DISPLAY *xdpy_create_display(int w, int h)
 
    ALLEGRO_DEBUG("X11 window created.\n");
    
-   set_size_hints(display, x_off, y_off);
+   _al_xwin_set_size_hints(display, x_off, y_off);
    
    XLockDisplay(system->x11display);
    
@@ -498,9 +450,9 @@ static ALLEGRO_DISPLAY *xdpy_create_display(int w, int h)
    if ((display->flags & ALLEGRO_FULLSCREEN_WINDOW)) {
       ALLEGRO_INFO("Toggling fullscreen flag for %d x %d window.\n",
          display->w, display->h);
-      reset_size_hints(display);
-      _al_xglx_set_fullscreen_window(display, 2);
-      set_size_hints(display, INT_MAX, INT_MAX);
+      _al_xwin_reset_size_hints(display);
+      _al_xwin_set_fullscreen_window(display, 2);
+      _al_xwin_set_size_hints(display, INT_MAX, INT_MAX);
 
       XWindowAttributes xwa;
       XGetWindowAttributes(system->x11display, d->window, &xwa);
@@ -516,7 +468,7 @@ static ALLEGRO_DISPLAY *xdpy_create_display(int w, int h)
       /* XXX compiz is quiky, can't seem to find a combination of hints that
        * make sure we are layerd over panels, and are positioned properly */
 
-      //_al_xglx_set_fullscreen_window(display, 1);
+      //_al_xwin_set_fullscreen_window(display, 1);
       _al_xglx_set_above(display, 1);
 
       _al_xglx_fullscreen_to_display(system, d);
@@ -582,7 +534,7 @@ static ALLEGRO_DISPLAY *xdpy_create_display(int w, int h)
 #endif
 
    if (display->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY])
-      setup_gl(display);
+      _al_ogl_setup_gl(display);
 
    /* vsync */
 
@@ -630,7 +582,6 @@ static ALLEGRO_DISPLAY *xdpy_create_display(int w, int h)
 }
 
 
-
 static void xdpy_destroy_display(ALLEGRO_DISPLAY *d)
 {
    ALLEGRO_SYSTEM_XGLX *s = (void *)al_get_system_driver();
@@ -819,6 +770,7 @@ static void xdpy_update_display_region(ALLEGRO_DISPLAY *d, int x, int y,
    xdpy_flip_display(d);
 }
 
+
 static bool xdpy_acknowledge_resize(ALLEGRO_DISPLAY *d)
 {
    ALLEGRO_SYSTEM_XGLX *system = (ALLEGRO_SYSTEM_XGLX *)al_get_system_driver();
@@ -847,8 +799,9 @@ static bool xdpy_acknowledge_resize(ALLEGRO_DISPLAY *d)
       /* No context yet means this is a stray call happening during
        * initialization.
        */
-      if (glx->context)
-         setup_gl(d);
+      if (glx->context) {
+         _al_ogl_setup_gl(d);
+      }
    }
 
    _al_mutex_unlock(&system->lock);
@@ -857,7 +810,6 @@ static bool xdpy_acknowledge_resize(ALLEGRO_DISPLAY *d)
 }
 
 
-
 /* Note: The system mutex must be locked (exactly once) so when we
  * wait for the condition variable it gets auto-unlocked. For a
  * nested lock that would not be the case.
@@ -920,7 +872,7 @@ static bool xdpy_resize_display(ALLEGRO_DISPLAY *d, int w, int h)
    }
 
    if (d->flags & ALLEGRO_FULLSCREEN) {
-      _al_xglx_set_fullscreen_window(d, 0);
+      _al_xwin_set_fullscreen_window(d, 0);
       if (!_al_xglx_fullscreen_set_mode(system, glx, w, h, 0, 0)) {
          ret = false;
          goto skip_resize;
@@ -940,13 +892,13 @@ static bool xdpy_resize_display(ALLEGRO_DISPLAY *d, int w, int h)
    for (; attempts >= 0; attempts--) {
       const int old_resize_count = glx->resize_count;
       ALLEGRO_DEBUG("calling XResizeWindow, attempts=%d\n", attempts);
-      reset_size_hints(d);
+      _al_xwin_reset_size_hints(d);
       glx->programmatic_resize = true;
       XResizeWindow(system->x11display, glx->window, w, h);
       _al_display_xglx_await_resize(d, old_resize_count,
          (d->flags & ALLEGRO_FULLSCREEN));
       glx->programmatic_resize = false;
-      set_size_hints(d, INT_MAX, INT_MAX);
+      _al_xwin_set_size_hints(d, INT_MAX, INT_MAX);
 
       if (d->w == w && d->h == h) {
          ret = true;
@@ -964,7 +916,7 @@ static bool xdpy_resize_display(ALLEGRO_DISPLAY *d, int w, int h)
 skip_resize:
 
    if (d->flags & ALLEGRO_FULLSCREEN) {
-      _al_xglx_set_fullscreen_window(d, 1);
+      _al_xwin_set_fullscreen_window(d, 1);
       _al_xglx_set_above(d, 1);
       _al_xglx_fullscreen_to_display(system, glx);
       ALLEGRO_DEBUG("xdpy: resize fullscreen?\n");
@@ -975,11 +927,8 @@ skip_resize:
 }
 
 
-
-/* Handle an X11 configure event. [X11 thread]
- * Only called from the event handler with the system locked.
- */
-void _al_display_xglx_configure(ALLEGRO_DISPLAY *d, XEvent *xevent)
+void _al_xglx_display_configure(ALLEGRO_DISPLAY *d, int x, int y,
+   int width, int height, bool setglxy)
 {
    ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)d;
 
@@ -992,16 +941,16 @@ void _al_display_xglx_configure(ALLEGRO_DISPLAY *d, XEvent *xevent)
     * Here we merely add the event to the queue.
     */
    if (!glx->programmatic_resize &&
-         (d->w != xevent->xconfigure.width ||
-          d->h != xevent->xconfigure.height)) {
+         (d->w != width ||
+          d->h != height)) {
       if (_al_event_source_needs_to_generate_event(es)) {
          ALLEGRO_EVENT event;
          event.display.type = ALLEGRO_EVENT_DISPLAY_RESIZE;
          event.display.timestamp = al_get_time();
-         event.display.x = xevent->xconfigure.x;
-         event.display.y = xevent->xconfigure.y;
-         event.display.width = xevent->xconfigure.width;
-         event.display.height = xevent->xconfigure.height;
+         event.display.x = x;
+         event.display.y = y;
+         event.display.width = width;
+         event.display.height = height;
          _al_event_source_emit_event(es, &event);
       }
    }
@@ -1012,20 +961,20 @@ void _al_display_xglx_configure(ALLEGRO_DISPLAY *d, XEvent *xevent)
     * Unfortunately, we also end up ignoring the only event we receive in
     * response to a XMoveWindow request so we have to compensate for that.
     */
-   if (xevent->xconfigure.send_event) {
-      glx->x = xevent->xconfigure.x;
-      glx->y = xevent->xconfigure.y;
+   if (setglxy) {
+      glx->x = x;
+      glx->y = y;
    }
    
 
    ALLEGRO_SYSTEM_XGLX *system = (ALLEGRO_SYSTEM_XGLX*)al_get_system_driver();
    ALLEGRO_MONITOR_INFO mi;
-   int center_x = (glx->x + (glx->x + xevent->xconfigure.width)) / 2;
-   int center_y = (glx->y + (glx->y + xevent->xconfigure.height)) / 2;
+   int center_x = (glx->x + (glx->x + width)) / 2;
+   int center_y = (glx->y + (glx->y + height)) / 2;
          
    _al_xglx_get_monitor_info(system, glx->adapter, &mi);
    
-   ALLEGRO_DEBUG("xconfigure event! %ix%i\n", xevent->xconfigure.x, xevent->xconfigure.y);
+   ALLEGRO_DEBUG("xconfigure event! %ix%i\n", x, y);
    
    /* check if we're no longer inside the stored adapter */
    if ((center_x < mi.x1 && center_x > mi.x2) ||
@@ -1045,28 +994,23 @@ void _al_display_xglx_configure(ALLEGRO_DISPLAY *d, XEvent *xevent)
    }
    
    _al_event_source_unlock(es);
-   
 }
 
 
-
-/* Handle an X11 close button event. [X11 thread]
+/* Handle an X11 configure event. [X11 thread]
  * Only called from the event handler with the system locked.
  */
-void _al_display_xglx_closebutton(ALLEGRO_DISPLAY *d, XEvent *xevent)
+void _al_xglx_display_configure_event(ALLEGRO_DISPLAY *d, XEvent *xevent)
 {
-   ALLEGRO_EVENT_SOURCE *es = &d->es;
-   (void)xevent;
-
-   _al_event_source_lock(es);
-
-   if (_al_event_source_needs_to_generate_event(es)) {
-      ALLEGRO_EVENT event;
-      event.display.type = ALLEGRO_EVENT_DISPLAY_CLOSE;
-      event.display.timestamp = al_get_time();
-      _al_event_source_emit_event(es, &event);
-   }
-   _al_event_source_unlock(es);
+   /* We receive two configure events when toggling the window frame.
+    * We ignore the first one as it has bogus coordinates.
+    * The only way to tell them apart seems to be the send_event field.
+    * Unfortunately, we also end up ignoring the only event we receive in
+    * response to a XMoveWindow request so we have to compensate for that.
+    */
+   bool setglxy = (xevent->xconfigure.send_event);
+   _al_xglx_display_configure(d, xevent->xconfigure.x, xevent->xconfigure.y,
+      xevent->xconfigure.width, xevent->xconfigure.height, setglxy);
 }
 
 
@@ -1157,6 +1101,10 @@ static void xdpy_set_window_title(ALLEGRO_DISPLAY *display, const char *title)
    XSetTextProperty(system->x11display, glx->window, &property, WM_NAME);
    XSetTextProperty(system->x11display, glx->window, &property, _NET_WM_NAME);
    XFree(property.value);
+   XClassHint hint;
+   hint.res_name = strdup(title); // Leak? (below too...)
+   hint.res_class = strdup(title);
+   XSetClassHint(system->x11display, glx->window, &hint);
    _al_mutex_unlock(&system->lock);
 }
 
@@ -1197,17 +1145,48 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_xglx_driver(void)
    xdpy_vt.set_target_bitmap = _al_ogl_set_target_bitmap;
    xdpy_vt.is_compatible_bitmap = xdpy_is_compatible_bitmap;
    xdpy_vt.resize_display = xdpy_resize_display;
-   xdpy_vt.set_icon = xdpy_set_icon;
+   xdpy_vt.set_icons = xdpy_set_icons;
    xdpy_vt.set_window_title = xdpy_set_window_title;
    xdpy_vt.set_window_position = xdpy_set_window_position;
    xdpy_vt.get_window_position = xdpy_get_window_position;
    xdpy_vt.set_display_flag = xdpy_set_display_flag;
    xdpy_vt.wait_for_vsync = xdpy_wait_for_vsync;
 
-   _al_xglx_add_cursor_functions(&xdpy_vt);
+   _al_xwin_add_cursor_functions(&xdpy_vt);
    _al_ogl_add_drawing_functions(&xdpy_vt);
 
    return &xdpy_vt;
 }
 
+#define X11_ATOM(x)  XInternAtom(x11, #x, False);
+
+void _al_xglx_set_above(ALLEGRO_DISPLAY *display, int value)
+{
+   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
+   ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
+   Display *x11 = system->x11display;
+
+   ALLEGRO_DEBUG("Toggling _NET_WM_STATE_ABOVE hint: %d\n", value);
+
+   XEvent xev;
+   xev.xclient.type = ClientMessage;
+   xev.xclient.serial = 0;
+   xev.xclient.send_event = True;
+   xev.xclient.message_type = X11_ATOM(_NET_WM_STATE);
+   xev.xclient.window = glx->window;
+   xev.xclient.format = 32;
+
+   // Note: It seems 0 is not reliable except when mapping a window -
+   // 2 is all we need though.
+   xev.xclient.data.l[0] = value; /* 0 = off, 1 = on, 2 = toggle */
+
+   xev.xclient.data.l[1] = X11_ATOM(_NET_WM_STATE_ABOVE);
+   xev.xclient.data.l[2] = 0;
+   xev.xclient.data.l[3] = 0;
+   xev.xclient.data.l[4] = 1;
+
+   XSendEvent(x11, DefaultRootWindow(x11), False,
+      SubstructureRedirectMask | SubstructureNotifyMask, &xev);
+}
+
 /* vi: set sts=3 sw=3 et: */
