@@ -1,5 +1,10 @@
+#include <X11/Xlib.h>
+
 #include "allegro5/allegro.h"
-#include "allegro5/internal/aintern_xglx.h"
+#include "allegro5/internal/aintern_x.h"
+#include "allegro5/internal/aintern_xdisplay.h"
+#include "allegro5/internal/aintern_xfullscreen.h"
+#include "allegro5/internal/aintern_xsystem.h"
 
 ALLEGRO_DEBUG_CHANNEL("display")
 
@@ -990,73 +995,4 @@ void _al_xglx_handle_mmon_event(ALLEGRO_SYSTEM_XGLX *s, ALLEGRO_DISPLAY_XGLX *d,
    _al_xglx_mmon_interface.handle_xevent(s, d, e);
 }
 
-#define X11_ATOM(x)  XInternAtom(x11, #x, False);
-
-/* Note: The system mutex must be locked (exactly once) before
- * calling this as we call _al_display_xglx_await_resize.
- */
-void _al_xglx_set_fullscreen_window(ALLEGRO_DISPLAY *display, int value)
-{
-   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
-   ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
-   Display *x11 = system->x11display;
-   int old_resize_count = glx->resize_count;
-
-   ALLEGRO_DEBUG("Toggling _NET_WM_STATE_FULLSCREEN hint: %d\n", value);
-
-   XEvent xev;
-   xev.xclient.type = ClientMessage;
-   xev.xclient.serial = 0;
-   xev.xclient.send_event = True;
-   xev.xclient.message_type = X11_ATOM(_NET_WM_STATE);
-   xev.xclient.window = glx->window;
-   xev.xclient.format = 32;
-
-   // Note: It seems 0 is not reliable except when mapping a window -
-   // 2 is all we need though.
-   xev.xclient.data.l[0] = value; /* 0 = off, 1 = on, 2 = toggle */
-
-   xev.xclient.data.l[1] = X11_ATOM(_NET_WM_STATE_FULLSCREEN);
-   xev.xclient.data.l[2] = 0;
-   xev.xclient.data.l[3] = 0;
-   xev.xclient.data.l[4] = 1;
-
-   XSendEvent(x11, RootWindowOfScreen(ScreenOfDisplay(x11, glx->xscreen)), False,
-      SubstructureRedirectMask | SubstructureNotifyMask, &xev);
-   
-   if (value == 2) {
-      /* Only wait for a resize if toggling. */
-      _al_display_xglx_await_resize(display, old_resize_count, true);
-   }
-}
-
-void _al_xglx_set_above(ALLEGRO_DISPLAY *display, int value)
-{
-   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
-   ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
-   Display *x11 = system->x11display;
-
-   ALLEGRO_DEBUG("Toggling _NET_WM_STATE_ABOVE hint: %d\n", value);
-
-   XEvent xev;
-   xev.xclient.type = ClientMessage;
-   xev.xclient.serial = 0;
-   xev.xclient.send_event = True;
-   xev.xclient.message_type = X11_ATOM(_NET_WM_STATE);
-   xev.xclient.window = glx->window;
-   xev.xclient.format = 32;
-
-   // Note: It seems 0 is not reliable except when mapping a window -
-   // 2 is all we need though.
-   xev.xclient.data.l[0] = value; /* 0 = off, 1 = on, 2 = toggle */
-
-   xev.xclient.data.l[1] = X11_ATOM(_NET_WM_STATE_ABOVE);
-   xev.xclient.data.l[2] = 0;
-   xev.xclient.data.l[3] = 0;
-   xev.xclient.data.l[4] = 1;
-
-   XSendEvent(x11, DefaultRootWindow(x11), False,
-      SubstructureRedirectMask | SubstructureNotifyMask, &xev);
-}
-
 /* vim: set sts=3 sw=3 et: */
