@@ -35,7 +35,10 @@
 #include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_events.h"
 #include "allegro5/internal/aintern_keyboard.h"
-#include "allegro5/internal/aintern_xglx.h"
+#include "allegro5/internal/aintern_x.h"
+#include "allegro5/internal/aintern_xdisplay.h"
+#include "allegro5/internal/aintern_xkeyboard.h"
+#include "allegro5/internal/aintern_xsystem.h"
 
 ALLEGRO_DEBUG_CHANNEL("keyboard")
 
@@ -446,24 +449,21 @@ void _al_xwin_keyboard_handler(XKeyEvent *event, ALLEGRO_DISPLAY *display)
 /* _al_xwin_keyboard_switch_handler:
  *  Handle a focus switch event.
  */
-void _al_xwin_keyboard_switch_handler(ALLEGRO_DISPLAY *display,
-   const XFocusChangeEvent *event)
+void _al_xwin_keyboard_switch_handler(ALLEGRO_DISPLAY *display, bool focus_in)
 {
    _al_event_source_lock(&the_keyboard.parent.es);
 
-   switch (event->type) {
-      case FocusIn:
-         the_keyboard.state.display = display;
+   if (focus_in) {
+      the_keyboard.state.display = display;
 #ifdef ALLEGRO_XWINDOWS_WITH_XIM
-         if (xic) {
-            ALLEGRO_DISPLAY_XGLX *display_glx = (void *)display;
-            XSetICValues(xic, XNClientWindow, display_glx->window, NULL);
-         }
+      if (xic) {
+         ALLEGRO_DISPLAY_XGLX *display_glx = (void *)display;
+         XSetICValues(xic, XNClientWindow, display_glx->window, NULL);
+      }
 #endif
-         break;
-      case FocusOut:
-         the_keyboard.state.display = NULL;
-         break;
+   }
+   else {
+      the_keyboard.state.display = NULL;
    }
 
    _al_event_source_unlock(&the_keyboard.parent.es);
@@ -515,7 +515,7 @@ static const char *x_scancode_to_name(int scancode)
  *  KeySyms to ALLEGRO_KEY_* codes.
  */
 
-static void _al_xwin_get_keyboard_mapping(void)
+static bool _al_xwin_get_keyboard_mapping(void)
 {
    int i;
    int count;
@@ -536,6 +536,10 @@ static void _al_xwin_get_keyboard_mapping(void)
 
    ALLEGRO_INFO("%i keys, %i symbols per key.\n", count, sym_per_key);
 
+   if (sym_per_key <= 0) {
+      return false;
+   }
+
    missing = 0;
 
    for (i = min_keycode; i <= max_keycode; i++) {
@@ -637,6 +641,8 @@ static void _al_xwin_get_keyboard_mapping(void)
       }
       key = al_get_next_config_entry(&it);
    }
+
+   return true;
 }
 
 
@@ -707,7 +713,13 @@ static int x_keyboard_init(void)
    ALLEGRO_INFO("Using X Input Method.\n");
 
    old_locale = setlocale(LC_CTYPE, NULL);
-   ALLEGRO_DEBUG("Old locale: %s\n", old_locale);
+   ALLEGRO_DEBUG("Old locale: %s\n", old_locale ? old_locale : "(null)");
+   if (old_locale) {
+      /* The return value of setlocale() may be clobbered by the next call
+       * to setlocale() so we must copy it.
+       */
+      old_locale = strdup(old_locale);
+   }
 
    /* Otherwise we are restricted to ISO-8859-1 characters. */
    if (setlocale(LC_CTYPE, "") == NULL) {
@@ -729,6 +741,7 @@ static int x_keyboard_init(void)
    if (old_locale) {
       ALLEGRO_DEBUG("Restoring old locale: %s\n", old_locale);
       setlocale(LC_CTYPE, old_locale);
+      free(old_locale);
    }
 
    if (xim) {
@@ -778,7 +791,9 @@ static int x_keyboard_init(void)
    }
 #endif
 
-   _al_xwin_get_keyboard_mapping();
+   if (!_al_xwin_get_keyboard_mapping()) {
+      return 1;
+   }
 
    _al_mutex_unlock(&s->lock);
 
