@@ -20,10 +20,16 @@
 
 #define ALLEGRO_NO_COMPATIBILITY
 
+#include <X11/Xlib.h>
+#include <stdio.h>
+
 #include "allegro5/allegro.h"
 #include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_mouse.h"
-#include "allegro5/internal/aintern_xglx.h"
+#include "allegro5/internal/aintern_x.h"
+#include "allegro5/internal/aintern_xdisplay.h"
+#include "allegro5/internal/aintern_xmouse.h"
+#include "allegro5/internal/aintern_xsystem.h"
 
 ALLEGRO_DEBUG_CHANNEL("mouse")
 
@@ -538,7 +544,38 @@ void _al_xwin_mouse_switch_handler(ALLEGRO_DISPLAY *display,
    _al_event_source_unlock(&the_mouse.parent.es);
 }
 
+bool _al_xwin_grab_mouse(ALLEGRO_DISPLAY *display)
+{
+   ALLEGRO_SYSTEM_XGLX *system = (ALLEGRO_SYSTEM_XGLX *)al_get_system_driver();
+   ALLEGRO_DISPLAY_XGLX *glx = (ALLEGRO_DISPLAY_XGLX *)display;
+   int grab;
+   bool ret;
 
+   _al_mutex_lock(&system->lock);
+   grab = XGrabPointer(system->x11display, glx->window, False,
+      PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
+      GrabModeAsync, GrabModeAsync, glx->window, None, CurrentTime);
+   if (grab == GrabSuccess) {
+      system->mouse_grab_display = display;
+      ret = true;
+   }
+   else {
+      ret = false;
+   }
+   _al_mutex_unlock(&system->lock);
+   return ret;
+}
+
+bool _al_xwin_ungrab_mouse(void)
+{
+   ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
+
+   _al_mutex_lock(&system->lock);
+   XUngrabPointer(system->x11display, CurrentTime);
+   system->mouse_grab_display = NULL;
+   _al_mutex_unlock(&system->lock);
+   return true;
+}
 
 /*
  * Local Variables:
