@@ -288,6 +288,10 @@ static int get_blend_factor(char const *value)
       : streq(value, "ALLEGRO_ONE") ? ALLEGRO_ONE
       : streq(value, "ALLEGRO_ALPHA") ? ALLEGRO_ALPHA
       : streq(value, "ALLEGRO_INVERSE_ALPHA") ? ALLEGRO_INVERSE_ALPHA
+      : streq(value, "ALLEGRO_SRC_COLOR") ? ALLEGRO_SRC_COLOR
+      : streq(value, "ALLEGRO_DEST_COLOR") ? ALLEGRO_DEST_COLOR
+      : streq(value, "ALLEGRO_INVERSE_SRC_COLOR") ? ALLEGRO_INVERSE_SRC_COLOR
+      : streq(value, "ALLEGRO_INVERSE_DEST_COLOR") ? ALLEGRO_INVERSE_DEST_COLOR
       : atoi(value);
 }
 
@@ -1183,6 +1187,9 @@ static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
       al_rest(delay);
    }
 
+   /* Ensure we don't target a bitmap which is about to be destroyed. */
+   al_set_target_bitmap(display ? al_get_backbuffer(display) : NULL);
+
    /* Destroy local bitmaps. */
    for (i = num_global_bitmaps; i < MAX_BITMAPS; i++) {
       if (bitmaps[i].name) {
@@ -1329,7 +1336,7 @@ static void process_ini_files(void)
 
    while (argc > 0) {
       if (!has_suffix(argv[0], ".ini"))
-         error("expected .ini arument: %s\n", argv[0]);
+         error("expected .ini argument: %s\n", argv[0]);
       cfg = al_load_config_file(argv[0]);
       if (!cfg)
          error("failed to load config file %s", argv[0]);
